@@ -3,11 +3,11 @@
 *  C4 dynamic programming library - code for heuristic pairs     *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -160,7 +160,7 @@ static C4_Score HPair_confirm_edge_func(gpointer src_data,
         if((HPair_check_entry(hpair, src_hsp,
                                hpair_edge_data->sar_join->region))
         || (HPair_check_exit(hpair, dst_hsp,
-                               hpair_edge_data->sar_join->region)))
+                             hpair_edge_data->sar_join->region)))
             return C4_IMPOSSIBLY_LOW_SCORE;
         score = SAR_Join_find_score(hpair_edge_data->sar_join,
                                     hpair);
@@ -168,7 +168,7 @@ static C4_Score HPair_confirm_edge_func(gpointer src_data,
         if((HPair_check_entry(hpair, src_hsp,
                                hpair_edge_data->sar_span->src_region))
         || (HPair_check_exit(hpair, dst_hsp,
-                              hpair_edge_data->sar_span->dst_region)))
+                             hpair_edge_data->sar_span->dst_region)))
             return C4_IMPOSSIBLY_LOW_SCORE;
         g_assert(hpair_edge_data->sar_span);
         score = SAR_Span_find_score(hpair_edge_data->sar_span, hpair);
@@ -194,7 +194,7 @@ static C4_Score HPair_update_edge_func(gpointer src_data,
         if((HPair_check_entry(hpair, src_hsp,
                                hpair_edge_data->sar_join->region))
         || (HPair_check_exit(hpair, dst_hsp,
-                               hpair_edge_data->sar_join->region)))
+                             hpair_edge_data->sar_join->region)))
             return C4_IMPOSSIBLY_LOW_SCORE;
         if(HPair_check_region_since(hpair,
                                      hpair_edge_data->sar_join->region,
@@ -207,7 +207,7 @@ static C4_Score HPair_update_edge_func(gpointer src_data,
         if((HPair_check_entry(hpair, src_hsp,
                                hpair_edge_data->sar_span->src_region))
         || (HPair_check_exit(hpair, dst_hsp,
-                              hpair_edge_data->sar_span->dst_region)))
+                             hpair_edge_data->sar_span->dst_region)))
             return C4_IMPOSSIBLY_LOW_SCORE;
         if((HPair_check_region_since(hpair,
                         hpair_edge_data->sar_span->src_region,
@@ -228,7 +228,7 @@ static C4_Score HPair_confirm_start_func(gpointer node_data,
     register HSP *hsp = HPair_NodeData_get_hsp(hpair, hpair_node_data);
     g_assert(hpair_node_data->sar_start);
     if(HPair_check_exit(hpair, hsp,
-                         hpair_node_data->sar_start->region))
+                        hpair_node_data->sar_start->region))
         return C4_IMPOSSIBLY_LOW_SCORE;
     return SAR_Terminal_find_score(hpair_node_data->sar_start,
                hpair_node_data->match->start_terminal->optimal,
@@ -244,7 +244,7 @@ static C4_Score HPair_update_start_func(gpointer node_data,
     register HSP *hsp = HPair_NodeData_get_hsp(hpair, hpair_node_data);
     g_assert(hpair_node_data->sar_start);
     if(HPair_check_exit(hpair, hsp,
-                         hpair_node_data->sar_start->region))
+                        hpair_node_data->sar_start->region))
         return C4_IMPOSSIBLY_LOW_SCORE;
     if(HPair_check_region_since(hpair,
                                 hpair_node_data->sar_start->region,
@@ -305,7 +305,7 @@ static void HPair_destroy_edge_data_func(gpointer data){
 
 /**/
 
-HPair *HPair_create(Heuristic *heuristic,
+HPair *HPair_create(Heuristic *heuristic, SubOpt *subopt,
                     gint query_length, gint target_length,
                     gint verbosity, gpointer user_data){
     register HPair *hpair = g_new(HPair, 1);
@@ -335,7 +335,7 @@ HPair *HPair_create(Heuristic *heuristic,
                               HPair_destroy_node_data_func,
                               HPair_destroy_edge_data_func,
                               hpair);
-    hpair->subopt = SubOpt_create(query_length, target_length);
+    hpair->subopt = SubOpt_share(subopt);
     return hpair;
     }
 
@@ -681,6 +681,7 @@ void HPair_finalise(HPair *hpair, C4_Score threshold){
 
 /**/
 
+#if 0
 static Alignment *HPair_refine_alignment(HPair *hpair,
                                          Alignment *alignment){
     register Alignment *refined_alignment = NULL;
@@ -732,9 +733,10 @@ static Alignment *HPair_refine_alignment(HPair *hpair,
     g_assert(refined_alignment->score >= alignment->score);
     return refined_alignment;
     }
+#endif /* 0 */
 
 Alignment *HPair_next_path(HPair *hpair, C4_Score threshold){
-    register Alignment *alignment, *refined_alignment;
+    register Alignment *alignment;
     register SAR_Alignment *sar_alignment;
     register BSDP_Path *bsdp_path;
     register BSDP_Node *first_node, *last_node,
@@ -786,6 +788,7 @@ Alignment *HPair_next_path(HPair *hpair, C4_Score threshold){
     BSDP_Path_destroy(bsdp_path);
     alignment = Alignment_share(sar_alignment->alignment);
     SAR_Alignment_destroy(sar_alignment);
+#if 0
     if(hpair->heuristic->has->refinement != Heuristic_Refinement_NONE){
         refined_alignment = HPair_refine_alignment(hpair, alignment);
         Alignment_destroy(alignment);
@@ -796,8 +799,11 @@ Alignment *HPair_next_path(HPair *hpair, C4_Score threshold){
             return NULL;
             }
         }
+#endif /* 0 */
     g_assert(alignment->score >= threshold);
+#if 0
     SubOpt_add_alignment(hpair->subopt, alignment);
+#endif /* 0 */
     return alignment;
     }
 
