@@ -3,11 +3,11 @@
 *  Library for HSP sets (high-scoring segment pairs)             *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -27,6 +27,12 @@ HSPset_ArgumentSet *HSPset_ArgumentSet_create(Argument *arg){
         ArgumentSet_add_option(as, '\0', "hspfilter", NULL,
                 "Aggressive HSP filtering level", "0",
                 Argument_parse_int, &has.filter_threshold);
+        ArgumentSet_add_option(as, '\0', "useworddropoff", NULL,
+                "Use word neighbourhood dropoff", "TRUE",
+                Argument_parse_boolean, &has.use_wordhood_dropoff);
+        ArgumentSet_add_option(as, '\0', "seedrepeat", NULL,
+                "Seeds per diagonal required for HSP seeding", "1",
+                Argument_parse_int, &has.seed_repeat);
         /**/
         ArgumentSet_add_option(as, 0, "dnawordlen", "bp",
             "Wordlength for DNA words", "12",
@@ -62,12 +68,19 @@ HSPset_ArgumentSet *HSPset_ArgumentSet_create(Argument *arg){
             "Score limit for dna word neighbourhood", "0",
             Argument_parse_int, &has.dna_wordlimit);
         ArgumentSet_add_option(as, 0, "proteinwordlimit", "score",
-            "Score limit for protein word neighbourhood", "3",
+            "Score limit for protein word neighbourhood", "4",
             Argument_parse_int, &has.protein_wordlimit);
         ArgumentSet_add_option(as, 0, "codonwordlimit", "score",
             "Score limit for codon word neighbourhood", "4",
             Argument_parse_int, &has.codon_wordlimit);
         /**/
+        ArgumentSet_add_option(as, '\0', "geneseed", "threshold",
+             "Geneseed Threshold", "0", 
+             Argument_parse_int, &has.geneseed_threshold);
+        ArgumentSet_add_option(as, '\0', "geneseedrepeat", "number",
+             "Seeds per diagonal required for geneseed HSP seeding", "2",
+             Argument_parse_int, &has.geneseed_repeat);
+        /**/
         Argument_absorb_ArgumentSet(arg, as);
         }
     return &has;
@@ -93,16 +106,63 @@ static gboolean HSP_check(HSP *hsp){
     return TRUE;
     }
 
+void HSP_Param_set_wordlen(HSP_Param *hsp_param, gint wordlen){
+    if(wordlen <= 0)
+        g_error("Wordlength must be greater than zero");
+    hsp_param->wordlen = wordlen;
+    hsp_param->seedlen = hsp_param->wordlen
+                       / hsp_param->match->query->advance;
+    return;
+    }
+
+void HSP_Param_set_dna_hsp_threshold(HSP_Param *hsp_param,
+                                     gint dna_hsp_threshold){
+    if(hsp_param->match->type == Match_Type_DNA2DNA)
+        hsp_param->threshold = dna_hsp_threshold;
+    return;
+    }
+
+void HSP_Param_set_protein_hsp_threshold(HSP_Param *hsp_param,
+                                         gint protein_hsp_threshold){
+    if((hsp_param->match->type == Match_Type_PROTEIN2PROTEIN)
+    || (hsp_param->match->type == Match_Type_PROTEIN2DNA)
+    || (hsp_param->match->type == Match_Type_DNA2PROTEIN))
+        hsp_param->threshold = protein_hsp_threshold;
+    return;
+    }
+
+void HSP_Param_set_codon_hsp_threshold(HSP_Param *hsp_param,
+                                       gint codon_hsp_threshold){
+    if(hsp_param->match->type == Match_Type_CODON2CODON)
+        hsp_param->threshold = codon_hsp_threshold;
+    return;
+    }
+
+void HSP_Param_set_hsp_threshold(HSP_Param *hsp_param,
+                                 gint hsp_threshold){
+    hsp_param->threshold = hsp_threshold;
+    return;
+    }
+
+void HSP_Param_set_seed_repeat(HSP_Param *hsp_param,
+                               gint seed_repeat){
+    hsp_param->seed_repeat = seed_repeat;
+    return;
+    }
+
 HSP_Param *HSP_Param_create(Match *match, gboolean use_horizon){
     register HSP_Param *hsp_param = g_new(HSP_Param, 1);
+    register WordHood_Alphabet *wha = NULL;
+    register Submat *submat;
     hsp_param->ref_count = 1;
     hsp_param->has = HSPset_ArgumentSet_create(NULL);
     hsp_param->match = match;
+    hsp_param->seed_repeat = hsp_param->has->seed_repeat;
     switch(match->type){
         case Match_Type_DNA2DNA:
             hsp_param->dropoff = hsp_param->has->dna_hsp_dropoff;
             hsp_param->threshold = hsp_param->has->dna_hsp_threshold;
-            hsp_param->wordlen = hsp_param->has->dna_wordlen;
+            HSP_Param_set_wordlen(hsp_param, hsp_param->has->dna_wordlen);
             hsp_param->wordlimit = hsp_param->has->dna_wordlimit;
             break;
         case Match_Type_PROTEIN2PROTEIN: /*fallthrough*/
@@ -111,15 +171,13 @@ HSP_Param *HSP_Param_create(Match *match, gboolean use_horizon){
             hsp_param->dropoff = hsp_param->has->protein_hsp_dropoff;
             hsp_param->threshold
                 = hsp_param->has->protein_hsp_threshold;
-            hsp_param->wordlen = hsp_param->has->protein_wordlen;
-            hsp_param->seedlen = hsp_param->wordlen;
+            HSP_Param_set_wordlen(hsp_param, hsp_param->has->protein_wordlen);
             hsp_param->wordlimit = hsp_param->has->protein_wordlimit;
             break;
         case Match_Type_CODON2CODON:
             hsp_param->dropoff = hsp_param->has->codon_hsp_dropoff;
             hsp_param->threshold = hsp_param->has->codon_hsp_threshold;
-            hsp_param->wordlen = hsp_param->has->codon_wordlen;
-            hsp_param->seedlen = hsp_param->wordlen / 3;
+            HSP_Param_set_wordlen(hsp_param, hsp_param->has->codon_wordlen);
             hsp_param->wordlimit = hsp_param->has->codon_wordlimit;
             g_assert(!(hsp_param->wordlen % 3));
             break;
@@ -127,9 +185,27 @@ HSP_Param *HSP_Param_create(Match *match, gboolean use_horizon){
             g_error("Bad Match_Type [%d]", match->type);
             break;
         }
-    hsp_param->seedlen = hsp_param->wordlen
-                       / hsp_param->match->query->advance;
     hsp_param->use_horizon = use_horizon;
+    if(hsp_param->has->use_wordhood_dropoff && (!hsp_param->wordlimit)){
+        hsp_param->wordhood = NULL;
+    } else {
+        if(match->type == Match_Type_CODON2CODON){
+            wha = WordHood_Alphabet_create_from_CodonSubmat(
+                                            match->mas->codon_submat, FALSE);
+        } else {
+            submat = (match->type == Match_Type_DNA2DNA)
+                   ? match->mas->dna_submat
+                   : match->mas->protein_submat;
+            wha = WordHood_Alphabet_create_from_Submat(
+                    (gchar*)match->comparison_alphabet->member,
+                    (gchar*)match->comparison_alphabet->member, submat, FALSE);
+            Submat_destroy(submat);
+            }
+        g_assert(wha);
+        hsp_param->wordhood = WordHood_create(wha, hsp_param->wordlimit,
+                                          hsp_param->has->use_wordhood_dropoff);
+        WordHood_Alphabet_destroy(wha);
+        }
     return hsp_param;
     }
 
@@ -137,6 +213,8 @@ void HSP_Param_destroy(HSP_Param *hsp_param){
     g_assert(hsp_param);
     if(--hsp_param->ref_count)
         return;
+    if(hsp_param->wordhood)
+        WordHood_destroy(hsp_param->wordhood);
     g_free(hsp_param);
     return;
     }
@@ -147,6 +225,12 @@ HSP_Param *HSP_Param_share(HSP_Param *hsp_param){
     return hsp_param;
     }
 
+HSP_Param *HSP_Param_swap(HSP_Param *hsp_param){
+    g_assert(hsp_param);
+    return HSP_Param_create(Match_swap(hsp_param->match),
+                            hsp_param->use_horizon);
+    }
+
 static RecycleBin *global_hsp_recycle_bin = NULL;
 
 HSPset *HSPset_create(Sequence *query, Sequence *target,
@@ -166,10 +250,12 @@ HSPset *HSPset_create(Sequence *query, Sequence *target,
         hsp_set->hsp_recycle = global_hsp_recycle_bin;
         }
     if(hsp_param->use_horizon){
-        hsp_set->horizon = (gint***)Matrix3d_create(query->len,
-                         hsp_param->match->query->advance,
-                         hsp_param->match->target->advance,
-                         sizeof(gint));
+        hsp_set->horizon = (gint****)Matrix4d_create(
+                                   1 + ((hsp_param->seed_repeat > 1)?1:0),
+                                   query->len,
+                                   hsp_param->match->query->advance,
+                                   hsp_param->match->target->advance,
+                                   sizeof(gint));
     } else {
         hsp_set->horizon = NULL;
         }
@@ -222,6 +308,50 @@ void HSPset_destroy(HSPset *hsp_set){
     return;
     }
 
+void HSPset_swap(HSPset *hsp_set, HSP_Param *hsp_param){
+    register Sequence *query;
+    register gint i, query_start;
+    register HSP *hsp;
+    g_assert(hsp_set->ref_count == 1);
+    /* Swap query and target */
+    query = hsp_set->query;
+    hsp_set->query = hsp_set->target;
+    hsp_set->target = query;
+    /* Switch parameters */
+    HSP_Param_destroy(hsp_set->param);
+    hsp_set->param = HSP_Param_share(hsp_param);
+    /* Swap HSPs coordinates */
+    for(i = 0; i < hsp_set->hsp_list->len; i++){
+        hsp = hsp_set->hsp_list->pdata[i];
+        query_start = hsp->query_start;
+        hsp->query_start = hsp->target_start;
+        hsp->target_start = query_start;
+        }
+    return;
+    }
+
+void HSPset_revcomp(HSPset *hsp_set){
+    register Sequence *rc_query = Sequence_revcomp(hsp_set->query),
+                      *rc_target = Sequence_revcomp(hsp_set->target);
+    register gint i;
+    register HSP *hsp;
+    g_assert(hsp_set);
+    g_assert(hsp_set->is_finalised);
+    g_assert(hsp_set->ref_count == 1);
+    /**/
+    Sequence_destroy(hsp_set->query);
+    Sequence_destroy(hsp_set->target);
+    hsp_set->query = rc_query;
+    hsp_set->target = rc_target;
+    for(i = 0; i < hsp_set->hsp_list->len; i++){
+        hsp = hsp_set->hsp_list->pdata[i];
+        hsp->query_start = hsp_set->query->len - HSP_query_end(hsp);
+        hsp->target_start = hsp_set->target->len - HSP_target_end(hsp);
+        }
+    return;
+    }
+
+
 static gint HSP_find_cobs(HSP *hsp){
     register gint i, query_pos = hsp->query_start,
                      target_pos = hsp->target_start;
@@ -674,7 +804,7 @@ static gboolean HSP_PQueue_comp_func(gpointer low, gpointer high,
 static void HSP_store(HSP *nascent_hsp){
     register HSPset *hsp_set = nascent_hsp->hsp_set;
     register PQueue *pq;
-    register HSP *hsp;
+    register HSP *hsp = NULL;
     g_assert(nascent_hsp);
     if(nascent_hsp->score < hsp_set->param->threshold)
         return;
@@ -718,7 +848,6 @@ static void HSP_store(HSP *nascent_hsp){
 
 void HSPset_seed_hsp(HSPset *hsp_set,
                      guint query_start, guint target_start){
-    HSP nascent_hsp;
     register gint diag_pos
         = ((target_start * hsp_set->param->match->query->advance)
           -(query_start * hsp_set->param->match->target->advance));
@@ -728,15 +857,22 @@ void HSPset_seed_hsp(HSPset *hsp_set,
                                % hsp_set->param->match->target->advance;
     register gint section_pos = (diag_pos + hsp_set->query->len)
                               % hsp_set->query->len;
+    HSP nascent_hsp;
     g_assert(!hsp_set->is_finalised);
     /**/
     g_assert(section_pos >= 0);
     g_assert(section_pos < hsp_set->query->len);
     /* Check whether we have seen this HSP already */
-    if(target_start < hsp_set->horizon[section_pos]
+    if(target_start < hsp_set->horizon[0]
+                                      [section_pos]
                                       [query_frame]
-                                      [target_frame]){
+                                      [target_frame])
         return;
+    if(hsp_set->param->seed_repeat > 1){
+        if(++hsp_set->horizon[1][section_pos][query_frame][target_frame]
+                < hsp_set->param->seed_repeat)
+            return;
+        hsp_set->horizon[1][section_pos][query_frame][target_frame] = 0;
         }
     /* Nascent HSP building: */
     nascent_hsp.hsp_set      = hsp_set;
@@ -753,15 +889,30 @@ void HSPset_seed_hsp(HSPset *hsp_set,
     || hsp_set->param->match->target->mask_func){
         HSP_extend(&nascent_hsp, TRUE);
         if(nascent_hsp.score < hsp_set->param->threshold){
-            hsp_set->horizon[section_pos][query_frame][target_frame]
+            hsp_set->horizon[0][section_pos][query_frame][target_frame]
                 = HSP_target_end(&nascent_hsp);
             return;
             }
         }
     /* Extend the HSP again ignoring masking */
     HSP_extend(&nascent_hsp, FALSE);
-    hsp_set->horizon[section_pos][query_frame][target_frame]
-        = HSP_target_end(&nascent_hsp);
+    HSP_store(&nascent_hsp);
+    hsp_set->horizon[0][section_pos][query_frame][target_frame]
+            = HSP_target_end(&nascent_hsp);
+    return;
+    }
+
+void HSPset_add_known_hsp(HSPset *hsp_set,
+                          guint query_start, guint target_start,
+                          guint length){
+    HSP nascent_hsp;
+    nascent_hsp.hsp_set      = hsp_set;
+    nascent_hsp.query_start  = query_start;
+    nascent_hsp.target_start = target_start;
+    nascent_hsp.length       = length;
+    nascent_hsp.cobs         = 0;
+    /* Score is irrelevant before HSP_init() */
+    HSP_init(&nascent_hsp);
     HSP_store(&nascent_hsp);
     return;
     }
@@ -787,10 +938,15 @@ static void HSPset_seed_hsp_sorted(HSPset *hsp_set,
     if(horizon[1][query_frame][target_frame] != section_pos){
         horizon[1][query_frame][target_frame] = section_pos;
         horizon[0][query_frame][target_frame] = 0;
+        horizon[2][query_frame][target_frame] = 0;
         }
+    if(++horizon[2][query_frame][target_frame] < hsp_set->param->seed_repeat)
+        return;
+    horizon[2][query_frame][target_frame] = 0;
     /* Check whether we have seen this HSP already */
     if(target_start < horizon[0][query_frame][target_frame])
         return;
+    /**/
     /* Nascent HSP building: */
     nascent_hsp.hsp_set      = hsp_set;
     nascent_hsp.query_start  = query_start;
@@ -798,10 +954,7 @@ static void HSPset_seed_hsp_sorted(HSPset *hsp_set,
     nascent_hsp.length       = hsp_set->param->seedlen;
     nascent_hsp.cobs         = 0;
     g_assert(HSP_check(&nascent_hsp));
-    /* Don't need to trim ends as (currently) only
-     * used for exact matching:
-     * HSP_trim_ends(&nascent_hsp);
-     */
+    HSP_trim_ends(&nascent_hsp);
     /* Score is irrelevant before HSP_init() */
     HSP_init(&nascent_hsp);
     /* Try to make above threshold HSP using masking */
@@ -816,11 +969,15 @@ static void HSPset_seed_hsp_sorted(HSPset *hsp_set,
         }
     /* Extend the HSP again ignoring masking */
     HSP_extend(&nascent_hsp, FALSE);
-    horizon[0][query_frame][target_frame]
-           = HSP_target_end(&nascent_hsp);
     HSP_store(&nascent_hsp);
+    /**/
+    horizon[0][query_frame][target_frame] = HSP_target_end(&nascent_hsp);
     return;
     }
+/* horizon[0] = diag
+ * horizon[1] = target_end
+ * horizon[2] = repeat_count
+ */
 
 /* Need to use the global to pass q,t advance to qsort compare func */
 static HSPset *HSPset_seed_compare_hsp_set = NULL;
@@ -852,15 +1009,15 @@ void HSPset_seed_all_hsps(HSPset *hsp_set,
         HSPset_seed_compare_hsp_set = NULL;
         }
     if(seed_list_len){
-        horizon = (gint***)Matrix3d_create(2,
+        horizon = (gint***)Matrix3d_create(3,
                      hsp_set->param->match->query->advance,
                      hsp_set->param->match->target->advance,
                      sizeof(gint));
         for(i = 0; i < seed_list_len; i++){
             HSPset_seed_hsp_sorted(hsp_set,
-                               seed_list[(i << 1)],
-                               seed_list[(i << 1) + 1],
-                               horizon);
+                                   seed_list[(i << 1)],
+                                   seed_list[(i << 1) + 1],
+                                   horizon);
             qpos = seed_list[(i << 1)];
             tpos = seed_list[(i << 1) + 1];
             }
@@ -869,9 +1026,6 @@ void HSPset_seed_all_hsps(HSPset *hsp_set,
     HSPset_finalise(hsp_set);
     return;
     }
-/* FIXME: optimisation: could bucket seeds on diag/frame
- *        and then sort the buckets ...
- */
 
 /**/
 
@@ -990,4 +1144,140 @@ void HSPset_filter_ungapped(HSPset *hsp_set){
     return;
     }
 
+/**/
+
+#define HSPset_SList_PAGE_BIT_WIDTH 12
+#define HSPset_SList_PAGE_SIZE (1 << HSPset_SList_PAGE_BIT_WIDTH)
+
+RecycleBin *HSPset_SList_RecycleBin_create(void){
+    return RecycleBin_create("HSPset_Slist", sizeof(HSPset_SList_Node),
+                             4096);
+    }
+
+HSPset_SList_Node *HSPset_SList_append(RecycleBin *recycle_bin,
+                                       HSPset_SList_Node *next,
+                                       gint query_pos, gint target_pos){
+    register HSPset_SList_Node *node = RecycleBin_alloc(recycle_bin);
+    node->next = next;
+    node->query_pos = query_pos;
+    node->target_pos = target_pos;
+    return node;
+    }
+
+void HSPset_seed_all_qy_sorted(HSPset *hsp_set, HSPset_SList_Node *seed_list){
+    register gint page_total = (hsp_set->target->len
+                                >> HSPset_SList_PAGE_BIT_WIDTH) + 1;
+    register HSPset_SList_Node **diag_page_list
+        = g_new0(HSPset_SList_Node*, page_total);
+    register gint i, page, diag_pos, query_frame, target_frame,
+                   section_pos, page_pos;
+    register HSPset_SList_Node *seed;
+    register gint *page_used = g_new(gint, page_total);
+    register gint page_used_total = 0;
+    register gint ****horizon = (gint****)Matrix4d_create(
+                           2 + ((hsp_set->param->seed_repeat > 1)?1:0),
+                           HSPset_SList_PAGE_SIZE,
+                           hsp_set->param->match->query->advance,
+                           hsp_set->param->match->target->advance,
+                           sizeof(gint));
+    HSP nascent_hsp;
+    /* Bin on diagonal into pages */
+    while(seed_list){
+        seed = seed_list;
+        seed_list = seed_list->next;
+        /**/
+        diag_pos = (seed->target_pos
+                    * hsp_set->param->match->query->advance)
+                 - (seed->query_pos
+                    * hsp_set->param->match->target->advance);
+        section_pos = ((diag_pos + hsp_set->target->len)
+                    % hsp_set->target->len);
+        page = (section_pos >> HSPset_SList_PAGE_BIT_WIDTH);
+        g_assert(section_pos >= 0);
+        g_assert(section_pos < hsp_set->target->len);
+        g_assert(page >= 0);
+        g_assert(page < page_total);
+        /**/
+        if(!diag_page_list[page])
+            page_used[page_used_total++] = page;
+        seed->next = diag_page_list[page];
+        diag_page_list[page] = seed;
+        }
+    /* Seed each page using page horizon */
+    for(i = 0; i < page_used_total; i++){
+        page = page_used[i];
+        for(seed = diag_page_list[page]; seed; seed = seed->next){
+            g_assert((!seed->next)
+                   || (seed->query_pos <= seed->next->query_pos));
+            diag_pos = (seed->target_pos
+                        * hsp_set->param->match->query->advance)
+                     - (seed->query_pos
+                        * hsp_set->param->match->target->advance);
+            query_frame  = seed->query_pos
+                         % hsp_set->param->match->query->advance;
+            target_frame  = seed->target_pos
+                          % hsp_set->param->match->target->advance;
+            section_pos = ((diag_pos + hsp_set->target->len)
+                        % hsp_set->target->len);
+            page_pos = section_pos
+                     - (page << HSPset_SList_PAGE_BIT_WIDTH);
+            g_assert(page_pos >= 0);
+            g_assert(page_pos < HSPset_SList_PAGE_SIZE);
+            /* Clear if page has changed */
+            if(horizon[1][page_pos][query_frame][target_frame] != page){
+               horizon[0][page_pos][query_frame][target_frame] = 0;
+               horizon[1][page_pos][query_frame][target_frame] = page;
+               if(hsp_set->param->seed_repeat > 1)
+                   horizon[2][page_pos][query_frame][target_frame] = 0;
+               }
+            if(seed->query_pos
+            < horizon[0][page_pos][query_frame][target_frame])
+                continue;
+            if(hsp_set->param->seed_repeat > 1){
+                if(++horizon[2][page_pos][query_frame][target_frame]
+                        < hsp_set->param->seed_repeat){
+                    continue;
+                    }
+                horizon[2][page_pos][query_frame][target_frame] = 0;
+                }
+            /* Nascent HSP building: */
+            nascent_hsp.hsp_set      = hsp_set;
+            nascent_hsp.query_start  = seed->query_pos;
+            nascent_hsp.target_start = seed->target_pos;
+            nascent_hsp.length       = hsp_set->param->seedlen;
+            nascent_hsp.cobs         = 0;
+            g_assert(HSP_check(&nascent_hsp));
+            HSP_trim_ends(&nascent_hsp);
+            /* Score is irrelevant before HSP_init() */
+            HSP_init(&nascent_hsp);
+            /* Try to make above threshold HSP using masking */
+            if(hsp_set->param->match->query->mask_func
+            || hsp_set->param->match->target->mask_func){
+                HSP_extend(&nascent_hsp, TRUE);
+                if(nascent_hsp.score < hsp_set->param->threshold){
+                    horizon[0][page_pos][query_frame][target_frame]
+                        = HSP_query_end(&nascent_hsp);
+                    continue;
+                    }
+                }
+            /* Extend the HSP again ignoring masking */
+            HSP_extend(&nascent_hsp, FALSE);
+            HSP_store(&nascent_hsp);
+            /**/
+            horizon[0][page_pos][query_frame][target_frame]
+                    = HSP_query_end(&nascent_hsp);
+            }
+        }
+    g_free(diag_page_list);
+    g_free(page_used);
+    g_free(horizon);
+    HSPset_finalise(hsp_set);
+    return;
+    }
+/* FIXME: move all allocs outside this function */
+/* horizon[horizon][mailbox][seed_repeat]
+ *        [page_size][qadv][tadv]
+ */
+
+/**/
 
