@@ -3,11 +3,11 @@
 *  Seeder : A module for seeding pairwise alignments             *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -36,11 +36,8 @@ Seeder_ArgumentSet *Seeder_ArgumentSet_create(Argument *arg){
         ArgumentSet_add_option(as, '\0', "forcefsm", "fsm type",
             "Force FSM type ( normal | compact )", "none",
             Argument_parse_string, &sas.force_fsm);
-        ArgumentSet_add_option(as, 0, "useworddropoff", NULL,
-            "Use word neighbourhood dropoff", "TRUE",
-            Argument_parse_boolean, &sas.use_wordhood_dropoff);
         ArgumentSet_add_option(as, 0, "wordjump", NULL,
-            "Jump betwen wordneighbourhood words", "1",
+            "Jump between query words", "1",
             Argument_parse_int, &sas.word_jump);
         Argument_absorb_ArgumentSet(arg, as);
         }
@@ -206,29 +203,12 @@ static Seeder_WordInfo *Seeder_add_WordInfo(Seeder *seeder, gchar *word,
     return word_info;
     }
 
-static gboolean Seeder_WordHood_traverse(gchar *word,
-                 gint score, gpointer user_data){
-    register Seeder_TraverseData *traverse_data = user_data;
-    register Seeder *seeder = traverse_data->seeder;
-    register Seeder_WordInfo *word_info;
-    g_assert(seeder);
-    word_info = Seeder_add_WordInfo(traverse_data->seeder, word,
-                                    traverse_data->context);
-    g_assert(word_info);
-    if(word_info == traverse_data->curr_word_info)
-        return FALSE; /* This is not a neighbour */
-    /* Make curr seed word a neighbour of this word */
-    Seeder_WordInfo_add_Neighbour(seeder, word_info,
-                                  traverse_data->curr_word_info);
-    return FALSE;
-    }
-
 static gboolean Seeder_word_is_valid(Match *match,
                                      Sequence *seq, gint pos,
                                      gint len){
     if(seq->annotation){
         if(match->type == Match_Type_DNA2DNA){
-            if(((pos+len) >= seq->annotation->cds_start)
+            if(((pos+len) > seq->annotation->cds_start)
             && (pos < (seq->annotation->cds_start
                       +seq->annotation->cds_length))){
                 return FALSE;
@@ -292,15 +272,12 @@ static gboolean Seeder_decide_fsm_type(gchar *force_fsm){
     }
 
 static Seeder_Loader *Seeder_Loader_create(HSP_Param *hsp_param,
-        Seeder_ArgumentSet *sas, gsize hspset_offset,
-        gboolean scan_query){
+        Seeder_ArgumentSet *sas, gsize hspset_offset){
     register Seeder_Loader *loader = g_new(Seeder_Loader, 1);
-    register WordHood_Alphabet *wha;
     register Alphabet *comparison_alphabet
                       = hsp_param->match->comparison_alphabet;
     register gint alphabet_size
               = strlen((gchar*)comparison_alphabet->member);
-    register Submat *submat;
     loader->hsp_param = HSP_Param_share(hsp_param);
     if(hsp_param->match->target->is_translated)
         loader->tpos_modifier = (hsp_param->wordlen * 3) - 3;
@@ -308,29 +285,12 @@ static Seeder_Loader *Seeder_Loader_create(HSP_Param *hsp_param,
         loader->tpos_modifier = hsp_param->wordlen - 1;
     loader->saturate_expectation = 1.0 / pow(alphabet_size,
                                              hsp_param->wordlen);
-    if(hsp_param->match->type == Match_Type_CODON2CODON){
-        wha = WordHood_Alphabet_create_from_CodonSubmat(
-                    hsp_param->match->mas->codon_submat, FALSE);
-    } else {
-        submat = (hsp_param->match->type == Match_Type_DNA2DNA)
-               ? hsp_param->match->mas->dna_submat
-               : hsp_param->match->mas->protein_submat;
-        wha = WordHood_Alphabet_create_from_Submat(
-                    (gchar*)comparison_alphabet->member,
-                    (gchar*)comparison_alphabet->member,
-                    submat, FALSE);
-        }
-    loader->wordhood = WordHood_create(wha, hsp_param->wordlimit,
-                            sas->use_wordhood_dropoff,
-                            Seeder_WordHood_traverse);
-    WordHood_Alphabet_destroy(wha);
     loader->hspset_offset = hspset_offset;
     return loader;
     }
 
 static void Seeder_Loader_destroy(Seeder_Loader *loader){
     HSP_Param_destroy(loader->hsp_param);
-    WordHood_destroy(loader->wordhood);
     g_free(loader);
     return;
     }
@@ -338,7 +298,6 @@ static void Seeder_Loader_destroy(Seeder_Loader *loader){
 Seeder *Seeder_create(gint verbosity,
                       Comparison_Param *comparison_param,
                       Match_Score saturate_threshold,
-                      gboolean scan_query,
                       Seeder_ReportFunc report_func,
                       gpointer user_data){
     register Seeder *seeder = g_new0(Seeder, 1);
@@ -350,9 +309,6 @@ Seeder *Seeder_create(gint verbosity,
     seeder->total_query_length = 0;
     seeder->comparison_count = 0;
     seeder->saturate_threshold = saturate_threshold;
-    seeder->scan_query = scan_query;
-    if(scan_query)
-        comparison_param = Comparison_Param_swap(comparison_param);
     seeder->comparison_param = Comparison_Param_share(comparison_param);
     seeder->query_info_list = g_ptr_array_new();
     if(saturate_threshold)
@@ -372,24 +328,21 @@ Seeder *Seeder_create(gint verbosity,
     if(comparison_param->dna_hsp_param){
         seeder->dna_loader
             = Seeder_Loader_create(comparison_param->dna_hsp_param,
-                seeder->sas, offsetof(Comparison, dna_hspset),
-                scan_query);
+                seeder->sas, offsetof(Comparison, dna_hspset));
         seeder->any_hsp_param
               = HSP_Param_share(comparison_param->dna_hsp_param);
         }
     if(comparison_param->protein_hsp_param){
         seeder->protein_loader
             = Seeder_Loader_create(comparison_param->protein_hsp_param,
-                seeder->sas, offsetof(Comparison, protein_hspset),
-                scan_query);
+                seeder->sas, offsetof(Comparison, protein_hspset));
         seeder->any_hsp_param
               = HSP_Param_share(comparison_param->protein_hsp_param);
         }
     if(comparison_param->codon_hsp_param){
         seeder->codon_loader
             = Seeder_Loader_create(comparison_param->codon_hsp_param,
-                seeder->sas, offsetof(Comparison, codon_hspset),
-                scan_query);
+                seeder->sas, offsetof(Comparison, codon_hspset));
         seeder->any_hsp_param
               = HSP_Param_share(comparison_param->codon_hsp_param);
         }
@@ -488,6 +441,23 @@ static gint Seeder_get_expect(Seeder *seeder, Seeder_Loader *loader,
                  + seeder->saturate_threshold);
     }
 
+static gboolean Seeder_WordHood_traverse(gchar *word,
+                 gint score, gpointer user_data){
+    register Seeder_TraverseData *traverse_data = user_data;
+    register Seeder *seeder = traverse_data->seeder;
+    register Seeder_WordInfo *word_info;
+    g_assert(seeder);
+    word_info = Seeder_add_WordInfo(traverse_data->seeder, word,
+                                    traverse_data->context);
+    g_assert(word_info);
+    if(word_info == traverse_data->curr_word_info)
+        return FALSE; /* This is not a neighbour */
+    /* Make curr seed word a neighbour of this word */
+    Seeder_WordInfo_add_Neighbour(seeder, word_info,
+                                  traverse_data->curr_word_info);
+    return FALSE;
+    }
+
 static void Seeder_insert_query(Seeder *seeder, Seeder_Context *context,
                                 Sequence *query, gint frame){
     register Match_Score query_expect = 0;
@@ -557,8 +527,10 @@ static void Seeder_insert_query(Seeder *seeder, Seeder_Context *context,
         if(word_info->seed_list /* not blocked */
         && (!word_info->seed_list->next)){ /* 1st seed */
             traverse_data.curr_word_info = word_info;
-            WordHood_traverse(context->loader->wordhood,
-                seq+pos, context->loader->hsp_param->wordlen, &traverse_data);
+            if(context->loader->hsp_param->wordhood)
+                WordHood_traverse(context->loader->hsp_param->wordhood,
+                              Seeder_WordHood_traverse, seq+pos,
+                              context->loader->hsp_param->wordlen, &traverse_data);
             }
         }
     Sequence_destroy(seq_masked);
@@ -643,12 +615,7 @@ static void Seeder_WordInfo_seed(Seeder_QueryInfo *query_info,
     else
         target_info->target_expect = 0;
     if(!query_info->curr_comparison){
-        if(target_info->seeder->scan_query)
-            query_info->curr_comparison = Comparison_create(
-                target_info->seeder->comparison_param,
-                target_info->target, query_info->query);
-        else
-            query_info->curr_comparison = Comparison_create(
+        query_info->curr_comparison = Comparison_create(
                 target_info->seeder->comparison_param,
                 query_info->query, target_info->target);
         g_ptr_array_add(target_info->seeder->active_queryinfo_list,
@@ -656,10 +623,7 @@ static void Seeder_WordInfo_seed(Seeder_QueryInfo *query_info,
         }
     hspset = OFFSET_ITEM(HSPset*, loader->hspset_offset,
                          query_info->curr_comparison);
-    if(target_info->seeder->scan_query)
-        HSPset_seed_hsp(hspset, target_pos, query_pos);
-    else
-        HSPset_seed_hsp(hspset, query_pos, target_pos);
+    HSPset_seed_hsp(hspset, query_pos, target_pos);
     return;
     }
 
@@ -799,7 +763,7 @@ void Seeder_add_target(Seeder *seeder, Sequence *target){
         if(Comparison_has_hsps(query_info->curr_comparison)){
             Comparison_finalise(query_info->curr_comparison);
             seeder->report_func(query_info->curr_comparison,
-                                    seeder->user_data);
+                                seeder->user_data);
             }
         Comparison_destroy(query_info->curr_comparison);
         query_info->curr_comparison = NULL;
