@@ -3,11 +3,11 @@
 *  Library for manipulation of exonerate dataset files           *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -20,27 +20,18 @@
 #include "fastadb.h"
 #include "bitarray.h"
 
-static void Dataset_write_int(guint64 num, FILE *fp){
-    guint64 nbo_num = GUINT64_TO_BE(num); /* BigEndian == NBO */
-    fwrite(&nbo_num, sizeof(guint64), 1, fp);
-    return;
-    }
-
-static guint64 Dataset_read_int(FILE *fp){
-    guint64 num;
-    fread(&num, sizeof(guint64), 1, fp);
-    return GUINT64_FROM_BE(num);
-    }
-
 /**/
 
-static Dataset_Header *Dataset_Header_create(gboolean is_dna,
+#define DATASET_HEADER_MAGIC (('e' << 16)|('s' << 8)|('d'))
+#define DATASET_HEADER_VERSION 1
+
+static Dataset_Header *Dataset_Header_create(Alphabet_Type alphabet_type,
                                              gboolean softmask_input){
     register Dataset_Header *header = g_new0(Dataset_Header, 1);
-    header->magic = 0; /* FIXME: pick magic number */
+    header->magic = DATASET_HEADER_MAGIC;
     header->version = 1;
-    header->type = (is_dna?1:0) | ((softmask_input?1:0) << 1);
-    g_message("made sm [%d] type [%d]", softmask_input, (gint)header->type);
+    header->type = ((alphabet_type == Alphabet_Type_DNA)?1:0)
+                 | ((softmask_input?1:0) << 1);
     /* other fields are filled during seq parsing or Dataset_finalise */
     return header;
     }
@@ -52,26 +43,23 @@ static void Dataset_Header_destroy(Dataset_Header *header){
 
 static void Dataset_Header_write(Dataset_Header *header, FILE *fp){
     /**/
-    Dataset_write_int(header->magic, fp);
-    Dataset_write_int(header->version, fp);
-    Dataset_write_int(header->type, fp);
-    Dataset_write_int(header->line_length, fp);
-    g_message("Write: magic [%d] version [%d] type [%d] line_len [%d]",
-            (gint)header->magic, (gint)header->version,
-            (gint)header->type, (gint)header->line_length);
+    BitArray_write_int(header->magic, fp);
+    BitArray_write_int(header->version, fp);
+    BitArray_write_int(header->type, fp);
+    BitArray_write_int(header->line_length, fp);
     /**/
-    Dataset_write_int(header->number_of_dbs, fp);
-    Dataset_write_int(header->max_db_len, fp);
-    Dataset_write_int(header->total_db_len, fp);
+    BitArray_write_int(header->number_of_dbs, fp);
+    BitArray_write_int(header->max_db_len, fp);
+    BitArray_write_int(header->total_db_len, fp);
     /**/
-    Dataset_write_int(header->number_of_seqs, fp);
-    Dataset_write_int(header->max_seq_len, fp);
-    Dataset_write_int(header->total_seq_len, fp);
+    BitArray_write_int(header->number_of_seqs, fp);
+    BitArray_write_int(header->max_seq_len, fp);
+    BitArray_write_int(header->total_seq_len, fp);
     /**/
-    Dataset_write_int(header->path_data_offset, fp);
-    Dataset_write_int(header->seq_info_offset, fp);
-    Dataset_write_int(header->seq_data_offset, fp);
-    Dataset_write_int(header->total_file_length, fp);
+    BitArray_write_int(header->path_data_offset, fp);
+    BitArray_write_int(header->seq_info_offset, fp);
+    BitArray_write_int(header->seq_data_offset, fp);
+    BitArray_write_int(header->total_file_length, fp);
     /**/
     return;
     }
@@ -79,30 +67,27 @@ static void Dataset_Header_write(Dataset_Header *header, FILE *fp){
 static Dataset_Header *Dataset_Header_read(FILE *fp){
     register Dataset_Header *header = g_new(Dataset_Header, 1);
     /**/
-    header->magic = Dataset_read_int(fp);
-    header->version = Dataset_read_int(fp);
-    header->type = Dataset_read_int(fp);
-    header->line_length = Dataset_read_int(fp);
-    g_message("Read: magic [%d] version [%d] type [%d] line_len [%d]",
-            (gint)header->magic, (gint)header->version,
-            (gint)header->type, (gint)header->line_length);
+    header->magic = BitArray_read_int(fp);
+    if(header->magic != DATASET_HEADER_MAGIC)
+        g_error("Bad magic number in dataset file");
+    header->version = BitArray_read_int(fp);
+    if(header->version != DATASET_HEADER_VERSION)
+        g_error("Incompatable dataset file version");
+    header->type = BitArray_read_int(fp);
+    header->line_length = BitArray_read_int(fp);
     /**/
-    header->number_of_dbs = Dataset_read_int(fp);
-    header->max_db_len = Dataset_read_int(fp);
-    header->total_db_len = Dataset_read_int(fp);
-    g_message("Have num dbs [%d], max db [%d], total [%d]",
-            (gint)header->number_of_dbs,
-            (gint)header->max_db_len,
-            (gint)header->total_db_len);
+    header->number_of_dbs = BitArray_read_int(fp);
+    header->max_db_len = BitArray_read_int(fp);
+    header->total_db_len = BitArray_read_int(fp);
     /**/
-    header->number_of_seqs = Dataset_read_int(fp);
-    header->max_seq_len = Dataset_read_int(fp);
-    header->total_seq_len = Dataset_read_int(fp);
+    header->number_of_seqs = BitArray_read_int(fp);
+    header->max_seq_len = BitArray_read_int(fp);
+    header->total_seq_len = BitArray_read_int(fp);
     /**/
-    header->path_data_offset = Dataset_read_int(fp);
-    header->seq_info_offset = Dataset_read_int(fp);
-    header->seq_data_offset = Dataset_read_int(fp);
-    header->total_file_length = Dataset_read_int(fp);
+    header->path_data_offset = BitArray_read_int(fp);
+    header->seq_info_offset = BitArray_read_int(fp);
+    header->seq_data_offset = BitArray_read_int(fp);
+    header->total_file_length = BitArray_read_int(fp);
     /**/
     return header;
     }
@@ -115,11 +100,13 @@ static Dataset_Sequence *Dataset_Sequence_create(FastaDB_Seq *fdbs){
     seq->gcg_checksum = Sequence_checksum(fdbs->seq);
     seq->id = g_strdup(fdbs->seq->id);
     seq->def = fdbs->seq->def?g_strdup(fdbs->seq->def):NULL;
-    g_message("set id [%s] def [%s]", seq->id, seq->def);
+    seq->cache_seq = NULL;
     return seq;
     }
 
 static void Dataset_Sequence_destroy(Dataset_Sequence *seq){
+    if(seq->cache_seq)
+        Sequence_destroy(seq->cache_seq);
     FastaDB_Key_destroy(seq->key);
     g_free(seq->id);
     if(seq->def)
@@ -128,12 +115,19 @@ static void Dataset_Sequence_destroy(Dataset_Sequence *seq){
     return;
     }
 
+static gsize Dataset_Sequence_memory_usage(Dataset_Sequence *ds){
+    return sizeof(Dataset_Sequence)
+         + sizeof(FastaDB_Key) /* FIXME: count FastaDB_Key memory properly */
+         + (sizeof(gchar)*strlen(ds->id))
+         + (sizeof(gchar)*(ds->def?strlen(ds->id):0))
+         + (ds->cache_seq?Sequence_memory_usage(ds->cache_seq):0);
+    }
+
 static int Dataset_Sequence_compare_by_id_uniq(const void *a,
                                           const void *b){
     register Dataset_Sequence **seq_a = (Dataset_Sequence**)a,
                               **seq_b = (Dataset_Sequence**)b;
     register gint retval = strcmp((*seq_a)->id, (*seq_b)->id);
-    g_message("compare [%s][%s]", (*seq_a)->id, (*seq_b)->id);
     if(!retval)
         g_error("Dataset has duplicate sequence id: [%s]",
                 (*seq_a)->id);
@@ -145,7 +139,6 @@ static int Dataset_Sequence_compare_by_id(const void *a,
     register Dataset_Sequence **seq_a = (Dataset_Sequence**)a,
                               **seq_b = (Dataset_Sequence**)b;
     register gint retval = strcmp((*seq_a)->id, (*seq_b)->id);
-    g_message("compare [%s][%s]", (*seq_a)->id, (*seq_b)->id);
     return retval;
     }
 
@@ -153,19 +146,13 @@ static int Dataset_Sequence_compare_by_id(const void *a,
 
 static Dataset_Width *Dataset_Width_create(Dataset_Header *header){
     register Dataset_Width *width = g_new0(Dataset_Width, 1);
-    register gint n;
-    g_message("A");
-    for(n = header->number_of_dbs; n; n >>= 1){
-        g_message("loop on n [%d]", n);
+    register guint64 n;
+    for(n = header->number_of_dbs; n; n >>= 1)
         width->num_db_width++;
-        }
-    g_message("B");
     for(n = header->max_db_len; n; n >>= 1)
         width->max_db_len_width++;
-    g_message("C");
     for(n = header->max_seq_len; n; n >>= 1)
         width->max_seq_len_width++;
-    g_message("D");
     n = width->num_db_width
       + width->max_db_len_width
       + width->max_seq_len_width
@@ -182,7 +169,7 @@ static void Dataset_Width_destroy(Dataset_Width *width){
 /**/
 
 Dataset *Dataset_create(GPtrArray *path_list,
-                        gboolean is_dna, gboolean softmask_input){
+                        Alphabet_Type alphabet_type, gboolean softmask_input){
     register Dataset *dataset = g_new(Dataset, 1);
     register FastaDB_Seq *fdbs;
     register Dataset_Sequence *ds;
@@ -190,15 +177,17 @@ Dataset *Dataset_create(GPtrArray *path_list,
     register gsize path_data_size, seq_info_size, seq_data_size;
     register gchar *path;
     dataset->ref_count = 1;
-    dataset->alphabet = Alphabet_create((is_dna?Alphabet_Type_DNA
-                                               :Alphabet_Type_PROTEIN),
-                                        softmask_input);
-    dataset->header = Dataset_Header_create(is_dna, softmask_input);
+    if(alphabet_type == Alphabet_Type_UNKNOWN){
+        alphabet_type = FastaDB_guess_type((gchar*)path_list->pdata[0]);
+        g_message("Guessed alphabet type as [%s]",
+                  Alphabet_Type_get_name(alphabet_type));
+        }
+    dataset->alphabet = Alphabet_create(alphabet_type, softmask_input);
+    dataset->header = Dataset_Header_create(alphabet_type, softmask_input);
     dataset->fdb = FastaDB_open_list(path_list, dataset->alphabet);
     /**/
     dataset->seq_list = g_ptr_array_new();
     while((fdbs = FastaDB_next(dataset->fdb, FastaDB_Mask_ALL))){
-        g_message("read seq [%s]", fdbs->seq->id);
         ds = Dataset_Sequence_create(fdbs);
         g_ptr_array_add(dataset->seq_list, ds);
         FastaDB_Seq_destroy(fdbs);
@@ -239,7 +228,6 @@ Dataset *Dataset_create(GPtrArray *path_list,
                                      + seq_info_size;
     dataset->header->total_file_length = dataset->header->seq_data_offset
                                        + seq_data_size;
-
     return dataset;
     }
 
@@ -248,6 +236,23 @@ Dataset *Dataset_share(Dataset *dataset){
     return dataset;
     }
 
+gsize Dataset_memory_usage(Dataset *dataset){
+    register gint i;
+    register gsize dataset_sequence_memory = 0;
+    register Dataset_Sequence *ds;
+    for(i = 0; i < dataset->seq_list->len; i++){
+        ds = dataset->seq_list->pdata[i];
+        dataset_sequence_memory += Dataset_Sequence_memory_usage(ds);
+        }
+    return sizeof(Dataset)
+         + sizeof(Alphabet)
+         + sizeof(Dataset_Header)
+         + sizeof(Dataset_Width)
+         + (sizeof(gpointer)*dataset->seq_list->len)
+         + FastaDB_memory_usage(dataset->fdb)
+         + dataset_sequence_memory;
+    }
+
 void Dataset_destroy(Dataset *dataset){
     register gint i;
     register Dataset_Sequence *seq;
@@ -267,6 +272,16 @@ void Dataset_destroy(Dataset *dataset){
     return;
     }
 
+gboolean Dataset_check_filetype(gchar *path){
+    register FILE *fp = fopen(path, "r");
+    register guint64 magic;
+    if(!fp)
+        g_error("Could not open file [%s]", path);
+    magic = BitArray_read_int(fp);
+    fclose(fp);
+    return (magic == DATASET_HEADER_MAGIC);
+    }
+
 static void Dataset_write_path_data(Dataset *dataset, FILE *fp){
     register gint i;
     register CompoundFile_Element *cfe;
@@ -312,7 +327,7 @@ static void Dataset_write_seq_data(Dataset *dataset, FILE *fp){
     }
 
 static void Dataset_read_seq_data(Dataset *dataset, FILE *fp){
-    register gint i;
+    register gint i, j;
     register Dataset_Sequence *ds;
     gchar buf[1024];
     for(i = 0; i < dataset->header->number_of_seqs; i++){
@@ -320,9 +335,15 @@ static void Dataset_read_seq_data(Dataset *dataset, FILE *fp){
         g_ptr_array_add(dataset->seq_list, ds);
         if(!fgets(buf, 1024, fp))
             g_error("Problem parsing seq data");
-        ds->id = g_strndup(buf, strlen(buf)-1);
-        g_message("read seqid [%s]", ds->id);
-        /* FIXME: should set ds->def when found */
+        for(j = 0; buf[j]; j++){
+            if(buf[j] == ' '){
+                ds->id = g_strndup(buf, j);
+                ds->def = g_strndup(buf+j+1, strlen(buf+j)-2);
+                break;
+                }
+            }
+        if(!ds->id)
+            ds->id = g_strndup(buf, strlen(buf)-1);
         }
     return;
     }
@@ -359,7 +380,6 @@ static void Dataset_read_seq_info(Dataset *dataset, FILE *fp){
         : Sequence_Strand_UNKNOWN;
     register CompoundFile_Location *location;
     for(i = 0; i < dataset->seq_list->len; i++){
-        g_message("reading on seq [%d]", i);
         ds = dataset->seq_list->pdata[i];
         ba = BitArray_read(fp, dataset->width->seq_data_item_size);
         start = 0;
@@ -409,9 +429,7 @@ Dataset *Dataset_read(gchar *path){
                             ?Alphabet_Type_DNA
                             :Alphabet_Type_PROTEIN),
                             (dataset->header->type&3));
-    g_message("get width");
     dataset->width = Dataset_Width_create(dataset->header);
-    g_message("got width");
     dataset->seq_list = g_ptr_array_new();
     Dataset_read_path_data(dataset, fp);
     Dataset_read_seq_data(dataset, fp);
@@ -439,21 +457,21 @@ Sequence *Dataset_get_sequence(Dataset *dataset, gint dataset_pos){
     register Dataset_Sequence *ds;
     register gchar *def;
     register SparseCache *cache;
-    register Sequence_Strand strand
-        = (dataset->alphabet->type == Alphabet_Type_DNA)
-        ? Sequence_Strand_FORWARD
-        : Sequence_Strand_UNKNOWN;
+    register Sequence_Strand strand;
     g_assert(dataset_pos >= 0);
     g_assert(dataset_pos < dataset->seq_list->len);
-    g_message("do db get seq");
     ds = dataset->seq_list->pdata[dataset_pos];
+    if(ds->cache_seq)
+        return Sequence_share(ds->cache_seq);
+    strand = (dataset->alphabet->type == Alphabet_Type_DNA)
+           ? Sequence_Strand_FORWARD
+           : Sequence_Strand_UNKNOWN;
     def = FastaDB_Key_get_def(ds->key);
     cache = FastaDB_Key_get_SparseCache(ds->key);
     seq = Sequence_create_extmem(ds->id, def, ds->key->length,
                                  strand, dataset->alphabet, cache);
     g_free(def);
     SparseCache_destroy(cache);
-    g_message("done db get seq");
     return seq;
     }
 /* FIXME: should keep a single compound file for whole dataset,
@@ -462,5 +480,42 @@ Sequence *Dataset_get_sequence(Dataset *dataset, gint dataset_pos){
  *        ?? will not work with cf sorting ... cf needs refactoring
  */
 
+static int Dataset_Sequence_preload_compare(const void *a, const void *b){
+    register Dataset_Sequence **ds_a = (Dataset_Sequence**)a,
+                              **ds_b = (Dataset_Sequence**)b;
+    g_assert(*ds_a);
+    g_assert(*ds_b);
+    g_assert((*ds_a)->key);
+    g_assert((*ds_b)->key);
+    if((*ds_a)->key->location->element_id
+    == (*ds_b)->key->location->element_id)
+        return (*ds_a)->key->location->pos
+             - (*ds_b)->key->location->pos;
+    return (*ds_a)->key->location->element_id
+         - (*ds_b)->key->location->element_id;
+    }
+
+void Dataset_preload_seqs(Dataset *dataset){
+    register gint i;
+    register Dataset_Sequence *ds;
+    register Dataset_Sequence **ds_list = g_new(Dataset_Sequence*,
+                                                dataset->header->number_of_seqs);
+    g_message("Preloading sequences");
+    for(i = 0; i < dataset->header->number_of_seqs; i++){
+        ds = dataset->seq_list->pdata[i];
+        ds->cache_seq = Dataset_get_sequence(dataset, i);
+        ds_list[i] = ds;
+        }
+    qsort(ds_list, dataset->header->number_of_seqs,
+          sizeof(Dataset_Sequence*), Dataset_Sequence_preload_compare);
+    /* Preload the sequence in the order they appear on disk */
+    for(i = 0; i < dataset->header->number_of_seqs; i++){
+        ds = ds_list[i];
+        Sequence_preload_extmem(ds->cache_seq);
+        }
+    g_free(ds_list);
+    return;
+    }
+
 /**/
 
