@@ -3,11 +3,11 @@
 *  Library for manipulation of FASTA format databases            *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -25,6 +25,10 @@
 
 #include "fastadb.h"
 
+#ifndef ALPHABET_SIZE
+#define ALPHABET_SIZE (sizeof(gchar)<<CHAR_BIT)
+#endif /* ALPHABET_SIZE */
+
 #define FASTADB_OUT_BUFFER_CHUNK_SIZE (BUFSIZ * 64)
 
 FastaDB_ArgumentSet *FastaDB_ArgumentSet_create(Argument *arg){
@@ -43,7 +47,7 @@ FastaDB_ArgumentSet *FastaDB_ArgumentSet_create(Argument *arg){
 static gboolean FastaDB_file_is_directory(gchar *path){
     struct stat buf;
     if(stat(path, &buf))
-        g_error("Cannot reading file [%s]", path);
+        g_error("Cannot read file [%s]", path);
     return S_ISDIR(buf.st_mode);
     }
 
@@ -254,6 +258,13 @@ void FastaDB_traverse(FastaDB *fdb, FastaDB_Mask mask,
     return;
     }
 
+gsize FastaDB_memory_usage(FastaDB *fdb){
+    return sizeof(FastaDB)
+         + sizeof(Alphabet)
+         + sizeof(CompoundFile)
+         + (sizeof(gchar)*fdb->out_buffer_alloc);
+    }
+
 /**/
 
 static FastaDB_Seq *FastaDB_Seq_create(FastaDB *fdb, Sequence *seq,
@@ -494,18 +505,192 @@ gchar *FastaDB_Key_get_def(FastaDB_Key *fdbk){
     while((ch = CompoundFile_getc(fdbk->source->cf)) != EOF){
         if(ch == '\n')
             break;
-        g_string_append_c(s, ch);
+        s = g_string_append_c(s, ch);
         }
     def = s->str;
     g_string_free(s, FALSE);
     return def;
     }
 
-static gpointer FastaDB_SparseCache_get_func(gint pos, gpointer page_data,
+/**/
+
+static gpointer FastaDB_SparseCache_get_func_8_BIT(gint pos, gpointer page_data,
                                              gpointer user_data){
     return GINT_TO_POINTER((gint)((gchar*)page_data)[pos]);
     }
 
+static gpointer FastaDB_SparseCache_get_func_0_BIT_LC(gint pos, gpointer page_data,
+                                                      gpointer user_data){
+    return GINT_TO_POINTER((gint)'n');
+    }
+
+static gpointer FastaDB_SparseCache_get_func_0_BIT_UC(gint pos, gpointer page_data,
+                                                      gpointer user_data){
+    return GINT_TO_POINTER((gint)'N');
+    }
+
+typedef enum {
+    FastaDB_SparseCache_Mask_0_BIT_LC, /* [n] */
+    FastaDB_SparseCache_Mask_0_BIT_UC, /* [N] */
+    FastaDB_SparseCache_Mask_2_BIT_LC, /* [acgt] */
+    FastaDB_SparseCache_Mask_2_BIT_UC, /* [ACGT] */
+    FastaDB_SparseCache_Mask_4_BIT,    /* [ACGTNacgtn] */
+    FastaDB_SparseCache_Mask_8_BIT
+} FastaDB_SparseCache_Mask;
+
+static void FastaDB_SparseCache_fill_mask(FastaDB_SparseCache_Mask *mask_index){
+    register gint i;
+    for(i = 0; i < ALPHABET_SIZE; i++)
+        mask_index[i] = (1 << FastaDB_SparseCache_Mask_8_BIT);
+    mask_index['N'] = (1 << FastaDB_SparseCache_Mask_0_BIT_UC);
+    mask_index['n'] = (1 << FastaDB_SparseCache_Mask_0_BIT_LC);
+    mask_index['A'] =
+    mask_index['C'] =
+    mask_index['G'] =
+    mask_index['T'] = (1 << FastaDB_SparseCache_Mask_2_BIT_UC);
+    mask_index['a'] =
+    mask_index['c'] =
+    mask_index['g'] =
+    mask_index['t'] = (1 << FastaDB_SparseCache_Mask_2_BIT_LC);
+    return;
+    }
+
+/**/
+
+static void FastaDB_SparseCache_fill_index(gint *index, gchar *alphabet){
+    register gint i;
+    for(i = 0; i < ALPHABET_SIZE; i++)
+        index[i] = -1;
+    for(i = 0; alphabet[i]; i++)
+        index[(guchar)alphabet[i]] = i;
+    return;
+    }
+
+static gpointer FastaDB_SparseCache_get_func_4_BIT(gint pos, gpointer page_data,
+                                                   gpointer user_data){
+    register gint ch = ((gchar*)page_data)[(pos >> 1)],
+                  num = (ch >> ((pos & 1) << 2)) & 15;
+    static gchar *alphabet = "ACGTNacgtn";
+    return GINT_TO_POINTER((gint)alphabet[num]);
+    }
+
+static void FastaDB_SparseCache_compress_4bit(SparseCache_Page *page, gint len){
+    register gint i;
+    register gchar *seq =  page->data;
+    static gint index[ALPHABET_SIZE];
+    static gboolean index_is_filled = FALSE;
+    if(!index_is_filled){
+        FastaDB_SparseCache_fill_index(index, "ACGTNacgtn");
+        index_is_filled = TRUE;
+        }
+    page->data = g_new0(gchar, (len >> 1)+1);
+    page->data_size = sizeof(gchar)*((len >> 1)+1);
+    for(i = 0; i < len; i++)
+        ((gchar*)page->data)[(i >> 1)] |= (index[(guchar)seq[i]] << ((i&1) << 2));
+    g_free(seq);
+    page->get_func = FastaDB_SparseCache_get_func_4_BIT;
+    return;
+    }
+
+/**/
+
+static gpointer FastaDB_SparseCache_get_func_2_BIT_LC(gint pos, gpointer page_data,
+                                                   gpointer user_data){
+    register gint ch = ((gchar*)page_data)[(pos >> 2)],
+                  num = (ch >> ((pos & 3) << 1)) & 3;
+    static gchar *alphabet = "acgt";
+    return GINT_TO_POINTER((gint)alphabet[num]);
+    }
+
+static void FastaDB_SparseCache_compress_2bit_lc(SparseCache_Page *page, gint len){
+    register gint i;
+    register gchar *seq =  page->data;
+    static gint index[ALPHABET_SIZE];
+    static gboolean index_is_filled = FALSE;
+    if(!index_is_filled){
+        FastaDB_SparseCache_fill_index(index, "acgt");
+        index_is_filled = TRUE;
+        }
+    page->data = g_new0(gchar, (len >> 2)+1);
+    page->data_size = sizeof(gchar)*((len >> 2)+1);
+    for(i = 0; i < len; i++)
+        ((gchar*)page->data)[(i >> 2)] |= (index[(guchar)seq[i]] << ((i&3) << 1));
+    g_free(seq);
+    page->get_func = FastaDB_SparseCache_get_func_2_BIT_LC;
+    return;
+    }
+
+static gpointer FastaDB_SparseCache_get_func_2_BIT_UC(gint pos, gpointer page_data,
+                                                   gpointer user_data){
+    register gint ch = ((gchar*)page_data)[(pos >> 2)],
+                  num = (ch >> ((pos & 3) << 1)) & 3;
+    static gchar *alphabet = "ACGT";
+    return GINT_TO_POINTER((gint)alphabet[num]);
+    }
+
+static void FastaDB_SparseCache_compress_2bit_uc(SparseCache_Page *page, gint len){
+    register gint i;
+    register gchar *seq =  page->data;
+    static gint index[ALPHABET_SIZE];
+    static gboolean index_is_filled = FALSE;
+    if(!index_is_filled){
+        FastaDB_SparseCache_fill_index(index, "ACGT");
+        index_is_filled = TRUE;
+        }
+    page->data = g_new0(gchar, (len >> 2)+1);
+    page->data_size = sizeof(gchar)*((len >> 2)+1);
+    for(i = 0; i < len; i++)
+        ((gchar*)page->data)[(i >> 2)] |= (index[(guchar)seq[i]] << ((i&3) << 1));
+    g_free(seq);
+    page->get_func = FastaDB_SparseCache_get_func_2_BIT_UC;
+    return;
+    }
+
+/**/
+
+void FastaDB_SparseCache_compress(SparseCache_Page *page, gint len){
+    register gint i, mask = 0;
+    register gchar *seq =  page->data;
+    static FastaDB_SparseCache_Mask mask_index[ALPHABET_SIZE];
+    static gboolean mask_is_filled = FALSE;
+    if(!mask_is_filled){
+        FastaDB_SparseCache_fill_mask(mask_index);
+        mask_is_filled = TRUE;
+        }
+    for(i = 0; i < len; i++)
+        mask |= mask_index[(guchar)seq[i]];
+    if(mask & (1 << FastaDB_SparseCache_Mask_8_BIT))
+        return; /* Keep current 8 bit encoding */
+    if(mask & (mask-1)){ /* more than one mask bit is set */
+        FastaDB_SparseCache_compress_4bit(page, len);
+        return;
+        }
+    if(mask & (1 << FastaDB_SparseCache_Mask_2_BIT_LC)){
+        FastaDB_SparseCache_compress_2bit_lc(page, len);
+        return;
+        }
+    if(mask & (1 << FastaDB_SparseCache_Mask_2_BIT_UC)){
+        FastaDB_SparseCache_compress_2bit_uc(page, len);
+        return;
+        }
+    if(mask & (1 << FastaDB_SparseCache_Mask_0_BIT_LC)){
+        g_free(page->data);
+        page->data = NULL;
+        page->data_size = 0;
+        page->get_func = FastaDB_SparseCache_get_func_0_BIT_LC;
+        return;
+        }
+    if(mask & (1 << FastaDB_SparseCache_Mask_0_BIT_UC)){
+        g_free(page->data);
+        page->data = NULL;
+        page->data_size = 0;
+        page->get_func = FastaDB_SparseCache_get_func_0_BIT_UC;
+        return;
+        }
+    g_error("Bad FastaDB_SparseCache compression mask");
+    return;
+    }
+
 static SparseCache_Page *FastaDB_SparseCache_fill_func(gint start,
                                                        gpointer user_data){
     register SparseCache_Page *page = g_new(SparseCache_Page, 1);
@@ -514,16 +699,19 @@ static SparseCache_Page *FastaDB_SparseCache_fill_func(gint start,
     register gint db_start, db_end;
     db_start = fdbk->seq_offset
              + start
-             + (start / (fdbk->source->line_length+1));
-    len = MIN(SparseCache_PAGE_SIZE, fdbk->length);
+             + ((fdbk->source->line_length == -1)
+               ?0:(start / (fdbk->source->line_length+1)));
+    len = MIN(SparseCache_PAGE_SIZE, fdbk->length-start);
     db_end = db_start
            + len
-           + (len / (fdbk->source->line_length+1));
+           + ((fdbk->source->line_length == -1)
+             ?0:(len / (fdbk->source->line_length+1)));
     fdbk->location->pos += db_start;
     CompoundFile_Location_seek(fdbk->location);
     fdbk->location->pos -= db_start;
     page->data = g_new(gchar, len);
-    page->get_func = FastaDB_SparseCache_get_func;
+    page->get_func = FastaDB_SparseCache_get_func_8_BIT;
+    page->data_size = sizeof(gchar)*len;
     do {
         ch = CompoundFile_getc(fdbk->source->cf);
         g_assert(ch != EOF);
@@ -531,6 +719,7 @@ static SparseCache_Page *FastaDB_SparseCache_fill_func(gint start,
         if(!isspace(ch))
             ((gchar*)page->data)[pos++] = ch;
     } while(pos < len);
+    FastaDB_SparseCache_compress(page, len);
     return page;
     }
 /* FIXME: this is really inefficient: should use unbuffered read/write
@@ -540,7 +729,7 @@ static SparseCache_Page *FastaDB_SparseCache_fill_func(gint start,
 
 SparseCache *FastaDB_Key_get_SparseCache(FastaDB_Key *fdbk){
     register SparseCache *cache = SparseCache_create(fdbk->length,
-                  FastaDB_SparseCache_fill_func, NULL, fdbk);
+                  FastaDB_SparseCache_fill_func, NULL, NULL, fdbk);
     return cache;
     }
 
@@ -611,22 +800,11 @@ Alphabet_Type FastaDB_guess_type(gchar *path){
     register Alphabet *alphabet = Alphabet_create(Alphabet_Type_UNKNOWN,
                                                   FALSE);
     register FastaDB_Seq *fdbs = FastaDB_get_single(path, alphabet);
-    static const guchar *index = (guchar*)
-    "----------------------------------------------------------------"
-    "-A-C---G------N-----T------------A-C---G------N-----T-----------"
-    "----------------------------------------------------------------"
-    "----------------------------------------------------------------";
-    register gint i, count = 0;
-    register Alphabet_Type type;
-    Alphabet_destroy(alphabet);
-    for(i = 0; i < fdbs->seq->len; i++)
-        if(index[Sequence_get_symbol(fdbs->seq, i)] != '-')
-            count++;
-    if((20.0*(gdouble)count) >= (17.0*(gdouble)fdbs->seq->len))
-        type = Alphabet_Type_DNA;
-    else
-        type = Alphabet_Type_PROTEIN;
+    register gchar *str = Sequence_get_str(fdbs->seq);
+    register Alphabet_Type type = Alphabet_Type_guess(str);
+    g_free(str);
     FastaDB_Seq_destroy(fdbs);
+    Alphabet_destroy(alphabet);
     return type;
     }
 
