@@ -3,11 +3,11 @@
 *  GAM: Gapped Alignment Manager                                 *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -35,6 +35,57 @@ static gchar *GAM_Argument_parse_Model_Type(gchar *arg_string,
     return NULL;
     }
 
+/**/
+
+static gchar *GAM_Refinement_to_string(GAM_Refinement refinement){
+    register gchar *name = NULL;
+    switch(refinement){
+        case GAM_Refinement_NONE:
+            name = "none";
+            break;
+        case GAM_Refinement_FULL:
+            name = "full";
+            break;
+        case GAM_Refinement_REGION:
+            name = "region";
+            break;
+        default:
+            g_error("Unknown GAM Refinement [%d]", refinement);
+            break;
+        }
+    return name;
+    }
+
+static GAM_Refinement GAM_Refinement_from_string(gchar *str){
+    gchar *name[GAM_Refinement_TOTAL] =
+        {"none", "full", "region"};
+    GAM_Refinement refinement[GAM_Refinement_TOTAL] =
+       {GAM_Refinement_NONE,
+        GAM_Refinement_FULL,
+        GAM_Refinement_REGION};
+    register gint i;
+    for(i = 0; i < GAM_Refinement_TOTAL; i++)
+        if(!g_strcasecmp(name[i], str))
+            return refinement[i];
+    g_error("Unknown refinement type [%s]", str);
+    return GAM_Refinement_NONE; /* Not reached */
+    }
+
+static gchar *GAM_Argument_parse_refinement(gchar *arg_string,
+                                                  gpointer data){
+    register GAM_Refinement *refinement
+          = (GAM_Refinement*)data;
+    gchar *refine_str;
+    register gchar *ret_val = Argument_parse_string(arg_string,
+                                                    &refine_str);
+    if(ret_val)
+        return ret_val;
+    (*refinement) = GAM_Refinement_from_string(refine_str);
+    return NULL;
+    }
+
+/**/
+
 GAM_ArgumentSet *GAM_ArgumentSet_create(Argument *arg){
     register ArgumentSet *as;
     static GAM_ArgumentSet gas;
@@ -46,6 +97,7 @@ GAM_ArgumentSet *GAM_ArgumentSet_create(Argument *arg){
         "    ungapped ungapped:trans\n"
         "    affine:global affine:bestfit affine:local affine:overlap\n"
         "    est2genome ner protein2dna protein2genome\n"
+        "    protein2dna:bestfit protein2genome:bestfit\n"
         "    coding2coding coding2genome cdna2genome genome2genome\n",
          "ungapped",
          GAM_Argument_parse_Model_Type, &gas.type);
@@ -85,9 +137,14 @@ GAM_ArgumentSet *GAM_ArgumentSet_create(Argument *arg){
         ArgumentSet_add_option(as, 'g', "gappedextension", NULL,
          "Use gapped extension (default is SDP)", "TRUE",
          Argument_parse_boolean, &gas.use_gapped_extension);
-        ArgumentSet_add_option(as, '\0', "geneseed", "threshold",
-         "Geneseed Threshold", "0",
-         Argument_parse_int, &gas.geneseed_threshold);
+        /**/
+        ArgumentSet_add_option(as, '\0', "refine", NULL,
+        "Alignment refinement strategy [none|full|region]", "none",
+        GAM_Argument_parse_refinement, &gas.refinement);
+        ArgumentSet_add_option(as, '\0', "refineboundary", NULL,
+        "Refinement region boundary", "32",
+        Argument_parse_int, &gas.refinement_boundary);
+        /**/
         Argument_absorb_ArgumentSet(arg, as);
         }
     return &gas;
@@ -105,8 +162,9 @@ static gboolean GAM_StoredResult_compare(gpointer low,
 
 static void GAM_display_alignment(GAM *gam,
             Alignment *alignment, Sequence *query, Sequence *target,
-            gint result_id, gint rank, gpointer user_data);
+            gint result_id, gint rank, gpointer user_data, FILE *fp);
 
+#if 0
 static FILE *GAM_bestn_tmp_file = NULL;
 
 static void GAM_StoredResult_print(const gchar *string){
@@ -114,19 +172,22 @@ static void GAM_StoredResult_print(const gchar *string){
     fprintf(GAM_bestn_tmp_file, "%s", string);
     return;
     }
+#endif /* 0 */
 
 static GAM_StoredResult *GAM_StoredResult_create(GAM *gam,
                          Sequence *query, Sequence *target,
                          Alignment *alignment, gpointer user_data){
     register GAM_StoredResult *gsr = g_new(GAM_StoredResult, 1);
-    register GPrintFunc prev_print_func;
+    /* register GPrintFunc prev_print_func; */
     gsr->score = alignment->score;
     gsr->pos = ftell(gam->bestn_tmp_file);
+    /*
     GAM_bestn_tmp_file = gam->bestn_tmp_file;
     prev_print_func = g_set_print_handler(GAM_StoredResult_print);
+    */
     GAM_display_alignment(gam, alignment, query, target,
-                          0, -1, user_data);
-    g_set_print_handler(prev_print_func);
+                          0, -1, user_data, gam->bestn_tmp_file);
+    /* g_set_print_handler(prev_print_func); */
     gsr->len = ftell(gam->bestn_tmp_file)
              - gsr->pos;
     return gsr;
@@ -381,13 +442,23 @@ GAM *GAM_create(Alphabet_Type query_type, Alphabet_Type target_type,
     gam->dual_match = Model_Type_has_dual_match(gam->gas->type);
     gam->model = Model_Type_get_model(gam->gas->type,
                                       query_type, target_type);
+    if((!use_exhaustive) && (!C4_Model_is_local(gam->model)))
+        g_error("Cannot perform heuristic alignment using non-local model");
     gam->match_list = GAM_build_match_list(gam->model);
     if(use_exhaustive){
         gam->optimal = Optimal_create(gam->model, NULL,
                        Optimal_Type_SCORE
                       |Optimal_Type_PATH
                       |Optimal_Type_REDUCED_SPACE, TRUE);
+        if(gam->gas->refinement != GAM_Refinement_NONE)
+            g_error("Exhaustive alignments cannot be refined");
     } else {
+        if(gam->gas->refinement != GAM_Refinement_NONE){
+            gam->optimal = Optimal_create(gam->model, NULL,
+                           Optimal_Type_SCORE
+                          |Optimal_Type_PATH
+                          |Optimal_Type_REDUCED_SPACE, TRUE);
+            }
         if(Model_Type_is_gapped(gam->gas->type)){
             if(gam->gas->use_gapped_extension){
                 gam->sdp = SDP_create(gam->model);
@@ -556,14 +627,79 @@ static GAM_Result *GAM_Result_create(GAM *gam,
     gam_result->target = Sequence_share(target);
     gam_result->user_data = Model_Type_create_data(gam->gas->type,
                                                    query, target);
+    gam_result->subopt = SubOpt_create(query->len, target->len);
     return gam_result;
     }
 
+static Alignment *GAM_Result_refine_alignment(GAM_Result *gam_result,
+                                              Alignment *alignment){
+    register Alignment *refined_alignment = NULL;
+    register Region *region;
+    register gint query_region_start, target_region_start;
+    g_assert(gam_result->gam->optimal);
+    if(gam_result->gam->verbosity > 1)
+        g_message("Refining alignment ... (%d)", alignment->score);
+    switch(gam_result->gam->gas->refinement){
+        case GAM_Refinement_FULL:
+            region = Region_create(0, 0, gam_result->query->len,
+                                         gam_result->target->len);
+            refined_alignment = Optimal_find_path(
+                    gam_result->gam->optimal, region,
+                    gam_result->user_data, alignment->score, gam_result->subopt);
+            g_assert(refined_alignment);
+            Region_destroy(region);
+            break;
+        case GAM_Refinement_REGION:
+            query_region_start
+                = MAX(0, alignment->region->query_start
+                       - gam_result->gam->gas->refinement_boundary);
+            target_region_start
+                = MAX(0, alignment->region->target_start
+                       - gam_result->gam->gas->refinement_boundary);
+            region = Region_create(query_region_start,
+                                   target_region_start,
+                MIN(gam_result->query->len,
+                    Region_query_end(alignment->region)
+                  + gam_result->gam->gas->refinement_boundary)
+                - query_region_start,
+                MIN(gam_result->target->len,
+                    Region_target_end(alignment->region)
+                  + gam_result->gam->gas->refinement_boundary)
+                - target_region_start);
+            refined_alignment = Optimal_find_path(
+                    gam_result->gam->optimal, region,
+                    gam_result->user_data, alignment->score, gam_result->subopt);
+            g_assert(refined_alignment);
+            Region_destroy(region);
+            break;
+        default:
+            g_error("Bad type for refinement [%s]",
+                GAM_Refinement_to_string(gam_result->gam->gas->refinement));
+            break;
+        }
+    g_assert(refined_alignment);
+    g_assert(refined_alignment->score >= alignment->score);
+    return refined_alignment;
+    }
+
 static void GAM_Result_add_alignment(GAM_Result *gam_result,
-                                     Alignment *alignment){
+                                     Alignment *alignment,
+                                     C4_Score threshold){
+    register Alignment *refined_alignment;
     if(!gam_result->alignment_list)
         gam_result->alignment_list = g_ptr_array_new();
     g_ptr_array_add(gam_result->alignment_list, alignment);
+    if(gam_result->gam->gas->refinement != GAM_Refinement_NONE){
+        refined_alignment = GAM_Result_refine_alignment(gam_result, alignment);
+        Alignment_destroy(alignment);
+        alignment = refined_alignment;
+        /* Refinement may put alignment below threshold */
+        if(alignment->score < threshold){
+            Alignment_destroy(alignment);
+            return;
+            }
+        }
+    SubOpt_add_alignment(gam_result->subopt, alignment);
     return;
     }
 
@@ -616,7 +752,7 @@ static void GAM_Result_ungapped_add_HSPset(GAM_Result *gam_result,
                                                   gam_result->user_data,
                                                   hsp);
             g_assert(alignment);
-            GAM_Result_add_alignment(gam_result, alignment);
+            GAM_Result_add_alignment(gam_result, alignment, threshold);
             }
         }
     return;
@@ -696,7 +832,7 @@ static GAM_Result *GAM_Result_BSDP_create(GAM *gam,
     if(gam->verbosity > 2)
         g_message("Preparing HPair for [%s][%s]",
                   comparison->query->id, comparison->target->id);
-    hpair = HPair_create(gam->heuristic,
+    hpair = HPair_create(gam->heuristic, gam_result->subopt,
                          comparison->query->len,
                          comparison->target->len,
                          gam->verbosity, gam_result->user_data);
@@ -723,7 +859,7 @@ static GAM_Result *GAM_Result_BSDP_create(GAM *gam,
         if(gam->verbosity > 2)
             g_message("Found BSDP alignment score [%d]",
                       alignment->score);
-        GAM_Result_add_alignment(gam_result, alignment);
+        GAM_Result_add_alignment(gam_result, alignment, threshold);
         if(GAM_Result_is_full(gam_result))
             break;
     } while(gam->gas->use_subopt);
@@ -753,8 +889,8 @@ static GAM_Result *GAM_Result_SDP_create(GAM *gam,
     g_assert(gam->sdp);
     g_assert(comparison);
     g_assert(Comparison_has_hsps(comparison));
-    sdp_pair = SDP_Pair_create(gam->sdp, comparison,
-                               gam_result->user_data);
+    sdp_pair = SDP_Pair_create(gam->sdp, gam_result->subopt,
+                               comparison, gam_result->user_data);
     do {
         threshold = GAM_get_query_threshold(gam, comparison->query);
         alignment = SDP_Pair_next_path(sdp_pair, threshold);
@@ -764,7 +900,7 @@ static GAM_Result *GAM_Result_SDP_create(GAM *gam,
         if(gam->verbosity > 2)
             g_message("Found SDP alignment score [%d]",
                       alignment->score);
-        GAM_Result_add_alignment(gam_result, alignment);
+        GAM_Result_add_alignment(gam_result, alignment, threshold);
         if(GAM_Result_is_full(gam_result))
             break;
     } while(gam->gas->use_subopt);
@@ -795,9 +931,11 @@ static void GAM_Result_geneseed_add(HSPset *hspset, GAM_Geneseed_Data *gsd){
         return;
     for(i = 0; i < hspset->hsp_list->len; i++){
         hsp = hspset->hsp_list->pdata[i];
-        RangeTree_add(gsd->rangetree,
-                HSP_query_cobs(hsp), HSP_target_cobs(hsp),
-                GINT_TO_POINTER(gsd->hsp_list->len));
+        if(!RangeTree_check_pos(gsd->rangetree,
+                                HSP_query_cobs(hsp), HSP_target_cobs(hsp)))
+            RangeTree_add(gsd->rangetree,
+                          HSP_query_cobs(hsp), HSP_target_cobs(hsp),
+                          GINT_TO_POINTER(gsd->hsp_list->len));
         g_ptr_array_add(gsd->hsp_list, hsp);
         if((!gsd->max_cobs_hsp) || (gsd->max_cobs_hsp->cobs < hsp->cobs))
                 gsd->max_cobs_hsp = hsp;
@@ -854,6 +992,7 @@ static void GAM_Result_geneseed_visit_hsp_fwd(GAM_Geneseed_Data *gsd,
                     (HSP_target_cobs(gsd->max_cobs_hsp)
                       - gsd->max_cobs_hsp->target_start),
                     query_range, target_range);
+        g_message("RangeTree_find fwd");
         */
         RangeTree_find(gsd->rangetree,
                        HSP_query_cobs(hsp), query_range,
@@ -890,6 +1029,7 @@ static void GAM_Result_geneseed_visit_hsp_rev(GAM_Geneseed_Data *gsd,
                     (HSP_target_cobs(gsd->max_cobs_hsp)
                       - gsd->max_cobs_hsp->target_start),
                     query_range, target_range);
+        g_message("RangeTree_find rev");
         */
         /* Search backwards */
         RangeTree_find(gsd->rangetree,
@@ -919,6 +1059,10 @@ static gint GAM_Result_geneseed_select(HSPset *hspset,
             }
         hsp_id++;
         }
+    /*
+    g_message("geneseed selected [%d] from [%d]", keep_list->len,
+                                                  hspset->hsp_list->len);
+    */
     g_ptr_array_free(hspset->hsp_list, TRUE);
     hspset->hsp_list = keep_list;
     return hsp_id;
@@ -928,7 +1072,6 @@ static void GAM_Result_geneseed_filter(GAM *gam, Comparison *comparison){
     register gint i, hsp_id = 0;
     register HSP *hsp;
     register C4_Span *span;
-    register GAM_ArgumentSet *gas = GAM_ArgumentSet_create(NULL);
     GAM_Geneseed_Data gsd;
     gsd.hsp_list = g_ptr_array_new();
     gsd.rangetree = RangeTree_create();
@@ -953,7 +1096,7 @@ static void GAM_Result_geneseed_filter(GAM *gam, Comparison *comparison){
     /* Find HSPs reachable from geneseed HSPs */
     for(i = 0; i < gsd.hsp_list->len; i++){
         hsp = gsd.hsp_list->pdata[i];
-        if(hsp->score >= gas->geneseed_threshold){
+        if(hsp->score >= hsp->hsp_set->param->has->geneseed_threshold){
             GAM_Result_geneseed_visit_hsp_fwd(&gsd, i);
             GAM_Result_geneseed_visit_hsp_rev(&gsd, i);
             }
@@ -998,12 +1141,25 @@ static void GAM_Result_geneseed_filter(GAM *gam, Comparison *comparison){
 
 GAM_Result *GAM_Result_heuristic_create(GAM *gam,
                                         Comparison *comparison){
-    GAM_Result_geneseed_filter(gam, comparison);
+    register GAM_Result *gam_result;
+    register HSPset_ArgumentSet *has
+        = Comparison_Param_get_HSPSet_Argument_Set(comparison->param);
+    if(has->geneseed_threshold)
+        GAM_Result_geneseed_filter(gam, comparison);
     if(!Comparison_has_hsps(comparison))
         return NULL;
+    /*
+    g_message("heuristic create with [%d,%d,%d]",
+        comparison->dna_hspset?comparison->dna_hspset->hsp_list->len:0,
+        comparison->protein_hspset?comparison->protein_hspset->hsp_list->len:0,
+        comparison->codon_hspset?comparison->codon_hspset->hsp_list->len:0);
+    Comparison_print(comparison);
+    */
     if(gam->gas->use_gapped_extension)
-        return GAM_Result_SDP_create(gam, comparison);
-    return GAM_Result_BSDP_create(gam, comparison);
+        gam_result = GAM_Result_SDP_create(gam, comparison);
+    else
+        gam_result = GAM_Result_BSDP_create(gam, comparison);
+    return gam_result;
     }
 
 /**/
@@ -1015,11 +1171,10 @@ GAM_Result *GAM_Result_exhaustive_create(GAM *gam,
     register C4_Score threshold;
     register GAM_Result *gam_result = GAM_Result_create(gam,
                                                         query, target);
-    register OPair *opair = OPair_create(gam->optimal,
-                                         query->len,
-                                         target->len,
-                                         gam_result->user_data);
+    register OPair *opair;
     g_assert(gam->optimal);
+    opair = OPair_create(gam->optimal, gam_result->subopt,
+                         query->len, target->len, gam_result->user_data);
     if(gam->verbosity > 1)
         g_message("Exhaustive alignment of [%s] [%s]",
                  query->id, target->id);
@@ -1030,7 +1185,7 @@ GAM_Result *GAM_Result_exhaustive_create(GAM *gam,
         if(!alignment)
             break;
         g_assert(alignment->score >= threshold);
-        GAM_Result_add_alignment(gam_result, alignment);
+        GAM_Result_add_alignment(gam_result, alignment, threshold);
         if(gam->verbosity > 2)
             g_message("Found alignment number [%d] score [%d]",
                 gam_result->alignment_list->len, alignment->score);
@@ -1062,33 +1217,35 @@ void GAM_Result_destroy(GAM_Result *gam_result){
     Sequence_destroy(gam_result->query);
     Sequence_destroy(gam_result->target);
     GAM_destroy(gam_result->gam);
+    SubOpt_destroy(gam_result->subopt);
     g_free(gam_result);
     return;
     }
 
 static void GAM_display_alignment(GAM *gam, Alignment *alignment,
             Sequence *query, Sequence *target,
-            gint result_id, gint rank, gpointer user_data){
+            gint result_id, gint rank, gpointer user_data, FILE *fp){
     if(gam->gas->show_alignment)
         Alignment_display(alignment, query, target,
                           gam->dna_submat, gam->protein_submat,
-                          gam->translate);
+                          gam->translate, fp);
     if(gam->gas->show_sugar)
-        Alignment_display_sugar(alignment, query, target);
+        Alignment_display_sugar(alignment, query, target, fp);
     if(gam->gas->show_cigar)
-        Alignment_display_cigar(alignment, query, target);
+        Alignment_display_cigar(alignment, query, target, fp);
     if(gam->gas->show_vulgar)
-        Alignment_display_vulgar(alignment, query, target);
+        Alignment_display_vulgar(alignment, query, target, fp);
     if(gam->gas->show_query_gff)
         Alignment_display_gff(alignment, query, target,
-                              TRUE, FALSE, result_id);
+                              TRUE, FALSE, result_id, fp);
     if(gam->gas->show_target_gff)
         Alignment_display_gff(alignment, query, target, FALSE,
-             Model_Type_has_genomic_target(gam->gas->type), result_id);
+             Model_Type_has_genomic_target(gam->gas->type), result_id, fp);
     if(gam->gas->ryo)
         Alignment_display_ryo(alignment, query, target,
                               gam->gas->ryo, gam->translate, rank,
-                              user_data);
+                              user_data, fp);
+    fflush(fp);
     return;
     }
 
@@ -1100,7 +1257,7 @@ static void GAM_Result_display(GAM_Result *gam_result){
         alignment = gam_result->alignment_list->pdata[i];
         GAM_display_alignment(gam_result->gam, alignment,
                 gam_result->query, gam_result->target,
-                i+1, 0, gam_result->user_data);
+                i+1, 0, gam_result->user_data, stdout);
         }
     return;
     }
