@@ -3,11 +3,11 @@
 *  Module of splice site and intron modelling                    *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -21,9 +21,6 @@ Intron_ArgumentSet *Intron_ArgumentSet_create(Argument *arg){
     static Intron_ArgumentSet ias;
     if(arg){
         as = ArgumentSet_create("Intron Modelling Options");
-        ArgumentSet_add_option(as, 0, "forcegtag", NULL,
-            "Force use of gt...ag splice sites", "FALSE",
-            Argument_parse_boolean, &ias.force_gtag);
         ArgumentSet_add_option(as, 0, "minintron", "length",
             "Minimum intron length", "30",
             Argument_parse_int, &ias.min_intron);
@@ -40,7 +37,7 @@ Intron_ArgumentSet *Intron_ArgumentSet_create(Argument *arg){
                       ias.intron_open_penalty);
     } else {
         if(!ias.sps)
-            ias.sps = SplicePredictorSet_create(ias.force_gtag);
+            ias.sps = SplicePredictorSet_create();
             /* FIXME: this should be freed somewhere */
         }
     return &ias;
