@@ -3,11 +3,11 @@
 *  C4 dynamic programming library - DP Scheduler                 *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -524,6 +524,9 @@ static SparseCache_Page *Scheduler_Cache_fill_func(gint start,
                          sizeof(Scheduler_SpanSeed*));
     register SparseCache_Page *page = g_new(SparseCache_Page, 1);
     page->data = seed_matrix;
+    page->data_size = Matrix2d_size(SparseCache_PAGE_SIZE,
+                                    spair->scheduler->model->span_list->len,
+                                    sizeof(Scheduler_SpanSeed*));
     page->get_func = Scheduler_Cache_get_func;
     return page;
     }
@@ -622,7 +625,7 @@ void Scheduler_SpanData_submit(Scheduler_SpanData *span_data,
         if(stored_seed){
             g_assert(stored_seed->query_entry
                   == span_seed->query_entry);
-            if(stored_seed->score < span_seed->score){
+            if(stored_seed->score <= span_seed->score){
                 Scheduler_SpanSeed_copy(span_seed, stored_seed, spair);
                 }
         } else {
@@ -1530,7 +1533,7 @@ Scheduler_Pair *Scheduler_Pair_create(
         spair->span_seed_cache = SparseCache_create(query_len+1,
                                                     Scheduler_Cache_fill_func,
                                                     Scheduler_Cache_empty_func,
-                                                    spair);
+                                                    NULL, spair);
         spair->span_history = Scheduler_SpanHistory_create(spair->slist_set,
                                                            scheduler->model);
     } else {
