@@ -3,11 +3,11 @@
 *  Simple Sequence Object                                        *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -115,13 +115,13 @@ Sequence_ArgumentSet *Sequence_ArgumentSet_create(Argument *arg){
     if(arg){
         as = ArgumentSet_create("Sequence Options");
         ArgumentSet_add_option(as, 'A', "annotation", "path",
-           "Path to sequence annotation file", "",
+           "Path to sequence annotation file", "none",
            Argument_parse_string, &sas.annotation_path);
         Argument_absorb_ArgumentSet(arg, as);
         Argument_add_cleanup(arg, Sequence_Argument_cleanup, &sas);
     } else {
         if(sas.annotation_path
-        && strlen(sas.annotation_path)
+        && strcmp(sas.annotation_path, "none")
         && (!sas.annotation_tree))
             sas.annotation_tree
                 = Sequence_create_annotation_tree(sas.annotation_path);
@@ -225,6 +225,15 @@ Sequence *Sequence_create_extmem(gchar *id, gchar *def, guint len,
     return s;
     }
 
+void Sequence_preload_extmem(Sequence *s){
+    register gint i;
+    if(s->type != Sequence_Type_EXTMEM)
+        return;
+    for(i = 0; i < s->len; i += SparseCache_PAGE_SIZE)
+        Sequence_get_symbol(s, i);
+    return;
+    }
+
 Sequence *Sequence_share(Sequence *s){
     g_assert(s);
     g_assert(s->ref_count);
@@ -477,39 +486,7 @@ gint Sequence_checksum(Sequence *s){
     return check % 10000; /* gcg checksum */
     }
 
-void Sequence_strncpy(Sequence *s, gint start, gint length, gchar *dst){
-    register gint i;
-    g_assert(start >= 0);
-    g_assert(s->len > 0);
-    g_assert(start < s->len);
-    g_assert((start+length) <= s->len);
-    /* FIXME: g_warning("using slow implementation of [%s]", __FUNCTION__); */
-    for(i = 0; i < length; i++)
-        dst[i] = Sequence_get_symbol(s, start+i);
-    return;
-    }
-/* FIXME: optimisation : implement fast versions
- *                       for specific sequence types
- */
-
-void Sequence_strcpy(Sequence *s, gchar *dst){
-    Sequence_strncpy(s, 0, s->len, dst);
-    return;
-    }
-
-gchar *Sequence_get_substr(Sequence *s, gint start, gint length){
-    register gchar *str = g_new(gchar, length+1);
-    Sequence_strncpy(s, start, length, str);
-    str[length] = '\0';
-    return str;
-    }
-
-gchar *Sequence_get_str(Sequence *s){
-    return Sequence_get_substr(s, 0, s->len);
-    }
-/* FIXME: optimisation: implement fast version
- *                      returning shared String for simple sequences
- */
+/**/
 
 typedef struct {
     Sequence *sequence;
@@ -562,6 +539,88 @@ Sequence *Sequence_translate(Sequence *s, Translate *translate, gint frame){
 
 /**/
 
+static void Sequence_print_type(Sequence *s){
+    register Sequence_Subseq *subseq;
+    register Sequence *revcomp;
+    register Sequence_Filter *filter;
+    register Sequence_Translation *translation;
+    switch(s->type){
+        case Sequence_Type_INTMEM:
+            g_print("intmem");
+            /* s->data is seq */
+            break;
+        case Sequence_Type_EXTMEM:
+            g_print("extmem");
+            /* s->data is SparseCache */
+            break;
+        case Sequence_Type_SUBSEQ:
+            g_print("subseq:");
+            subseq = s->data;
+            Sequence_print_type(subseq->sequence);
+            break;
+        case Sequence_Type_REVCOMP:
+            g_print("revcomp:");
+            revcomp = s->data;
+            Sequence_print_type(revcomp);
+            break;
+        case Sequence_Type_FILTER:
+            g_print("filter:");
+            filter = s->data;
+            Sequence_print_type(filter->sequence);
+            break;
+        case Sequence_Type_TRANSLATE:
+            g_print("translate:");
+            translation = s->data;
+            Sequence_print_type(translation->sequence);
+            break;
+        default:
+            g_error("unknown sequence type [%d]", s->type);
+            break;
+        }
+    return;
+    }
+
+void Sequence_strncpy(Sequence *s, gint start, gint length, gchar *dst){
+    register gint i;
+    g_assert(start >= 0);
+    g_assert(s->len > 0);
+    g_assert(start < s->len);
+    g_assert((start+length) <= s->len);
+    /* FIXME: g_warning("using slow implementation of [%s]", __FUNCTION__); */
+#if 0
+    g_print("Sequence_strncpy [");
+    Sequence_print_type(s);
+    g_print("] [%d] [%d] [%s]\n", start, length, s->id);
+#endif /* 0 */
+    for(i = 0; i < length; i++)
+        dst[i] = Sequence_get_symbol(s, start+i);
+    return;
+    }
+/* FIXME: optimisation : implement fast versions
+ *                       for specific sequence types
+ */
+
+void Sequence_strcpy(Sequence *s, gchar *dst){
+    Sequence_strncpy(s, 0, s->len, dst);
+    return;
+    }
+
+gchar *Sequence_get_substr(Sequence *s, gint start, gint length){
+    register gchar *str = g_new(gchar, length+1);
+    Sequence_strncpy(s, start, length, str);
+    str[length] = '\0';
+    return str;
+    }
+
+gchar *Sequence_get_str(Sequence *s){
+    return Sequence_get_substr(s, 0, s->len);
+    }
+/* FIXME: optimisation: implement fast version
+ *                      returning shared String for simple sequences
+ */
+
+/**/
+
 void Sequence_destroy(Sequence *s){
     g_assert(s);
     if(--s->ref_count)
@@ -678,3 +737,26 @@ Sequence *Sequence_mask(Sequence *s){
     return new_seq;
     }
 
+gsize Sequence_memory_usage(Sequence *s){
+    register SparseCache *cache;
+    register gsize data_memory = 0;
+    switch(s->type){
+        case Sequence_Type_INTMEM:
+            data_memory = sizeof(gchar)*s->len;
+            break;
+        case Sequence_Type_EXTMEM:
+            cache = s->data;
+            data_memory = SparseCache_memory_usage(cache);
+            break;
+        default:
+            data_memory = 0;
+            break;
+        }
+    return sizeof(Sequence)
+         + sizeof(gchar)*strlen(s->id)
+         + sizeof(gchar)*(s->def?strlen(s->def):0)
+         + sizeof(Alphabet)
+         + data_memory;
+    }
+
+
