@@ -3,11 +3,11 @@
 *  Library for Splice site prediction.                           *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -15,16 +15,37 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <math.h>  /* For log() */
-#include <ctype.h> /* For toupper() */
+#include <math.h>   /* For log() */
+#include <ctype.h>  /* For toupper() */
+#include <string.h> /* For strlen() */
 
 #include "splice.h"
 #include "matrix.h"
+#include "lineparse.h"
 
 #ifndef SPLICE_IMPOSSIBLY_LOW_SCORE
 #define SPLICE_IMPOSSIBLY_LOW_SCORE -987654321.0
 #endif /* SPLICE_IMPOSSIBLY_LOW_SCORE */
 
+Splice_ArgumentSet *Splice_ArgumentSet_create(Argument *arg){
+    register ArgumentSet *as;
+    static Splice_ArgumentSet sas = {NULL, NULL, FALSE};
+    if(arg){
+        as = ArgumentSet_create("Splice Site Prediction Options");
+        ArgumentSet_add_option(as, 0, "splice3", NULL,
+                "Supply frequency matrix for 3' splice sites", "primate",
+                Argument_parse_string, &sas.splice3_data_path);
+        ArgumentSet_add_option(as, 0, "splice5", NULL,
+                "Supply frequency matrix for 5' splice sites", "primate",
+                Argument_parse_string, &sas.splice5_data_path);
+        ArgumentSet_add_option(as, 0, "forcegtag", NULL,
+                "Force use of gt...ag splice sites", "FALSE",
+                Argument_parse_boolean, &sas.force_gtag);
+        Argument_absorb_ArgumentSet(arg, as);
+        }
+    return &sas;
+    }
+
 /* ------------------------------------------------------------ */
 /* --- Start of Splice data ----------------------------------- */
 
@@ -85,7 +106,7 @@ static void SplicePredictor_add_data_primate_3SS(SplicePredictor *sp){
     /*   1 */ {  28,  14,  47,  11 }
     };
     sp->model_length = 15;
-    sp->model_splice_after = 12;
+    sp->model_splice_after = 14;
     sp->model_data = (gfloat**)Matrix2d_create(sp->model_length,
                                                5, sizeof(gfloat));
     for(i = 0; i < sp->model_length; i++)
@@ -98,6 +119,97 @@ static void SplicePredictor_add_data_primate_3SS(SplicePredictor *sp){
 /* --- End of Splice data ------------------------------------- */
 /* ------------------------------------------------------------ */
 
+static void SplicePredictor_parse_data(SplicePredictor *sp, gchar *path){
+    register FILE *fp = fopen(path, "r");
+    register LineParse *lp;
+    register gint i;
+    gfloat num;
+    register gchar *word;
+    register GArray *number_list = g_array_new(FALSE, FALSE, sizeof(gfloat));
+    if(!fp)
+        g_error("Could not open splice frequency data [%s]", path);
+    lp = LineParse_create(fp);
+    /**/
+    sp->model_length = 0;
+    while(LineParse_word(lp) != EOF){
+        if(lp->word->len){
+            word = lp->word->pdata[0];
+            if(word[0] == '#'){
+                continue;
+                }
+            }
+        switch(lp->word->len){
+            case 0: /* ignore */
+                break;
+            case 1:
+                word = lp->word->pdata[0];
+                if(!g_strcasecmp(word, "splice"))
+                    sp->model_splice_after = sp->model_length;
+                else
+                    g_error("bad line in splice data file");
+                break;
+            case 4:
+                for(i = 0; i < 4; i++){
+                    num = atof(lp->word->pdata[i]);
+                    g_array_append_val(number_list, num);
+                    }
+                sp->model_length++;
+                break;
+            default:
+                g_error("bad line in splice data file");
+                break;
+            }
+        }
+    sp->model_data = NULL;
+    LineParse_destroy(lp);
+    fclose(fp);
+    sp->model_data = (gfloat**)Matrix2d_create(sp->model_length,
+                                               5, sizeof(gfloat));
+    for(i = 0; i < sp->model_length; i++){
+        sp->model_data[i][0] = g_array_index(number_list, gfloat, (i*4));
+        sp->model_data[i][1] = g_array_index(number_list, gfloat, (i*4)+1);
+        sp->model_data[i][2] = g_array_index(number_list, gfloat, (i*4)+2);
+        sp->model_data[i][3] = g_array_index(number_list, gfloat, (i*4)+3);
+        }
+    g_array_free(number_list, TRUE);
+    return;
+    }
+
+#if 0
+static void SplicePredictor_info(SplicePredictor *sp){
+    register gint i;
+    g_message("type [%d] len [%d] splice_after [%d]",
+            sp->type, sp->model_length, sp->model_splice_after);
+    for(i = 0; i < sp->model_length; i++)
+        g_message("    [%d] [%2.2f][%2.2f][%2.2f][%2.2f]",
+                i,
+                sp->model_data[i][0],
+                sp->model_data[i][1],
+                sp->model_data[i][2],
+                sp->model_data[i][3]);
+    return;
+    }
+#endif /* 0 */
+
+static void SplicePredictor_add_5SS_data(SplicePredictor *sp, gchar *path){
+    if((!path) || (!strlen(path)) || (!g_strcasecmp(path, "primate")))
+        SplicePredictor_add_data_primate_5SS(sp);
+    else
+        SplicePredictor_parse_data(sp, path);
+    return;
+    }
+
+static void SplicePredictor_add_3SS_data(SplicePredictor *sp, gchar *path){
+    if((!path) || (!strlen(path)) || (!g_strcasecmp(path, "primate")))
+        SplicePredictor_add_data_primate_3SS(sp);
+    else
+        SplicePredictor_parse_data(sp, path);
+    sp->model_splice_after -= 2;
+    return;
+    }
+
+/**/
+
 static SplicePredictor_GTAGonly *SplicePredictor_GTAGonly_create(
                                  SplicePredictor *sp){
     register SplicePredictor_GTAGonly *spgo
@@ -126,18 +238,19 @@ static SplicePredictor_GTAGonly *SplicePredictor_GTAGonly_create(
     return spgo;
     }
 
-SplicePredictor *SplicePredictor_create(SpliceType type,
-                                        gboolean gtag_only){
+
+SplicePredictor *SplicePredictor_create(SpliceType type){
     register SplicePredictor *sp = g_new(SplicePredictor, 1);
     register gint i, j, a, z;
     register gfloat swap;
+    register Splice_ArgumentSet *sas = Splice_ArgumentSet_create(NULL);
     sp->ref_count = 1;
     sp->type = type;
     if((type == SpliceType_ss5_forward)
     || (type == SpliceType_ss5_reverse)){
-        SplicePredictor_add_data_primate_5SS(sp);
+        SplicePredictor_add_5SS_data(sp, sas->splice5_data_path);
     } else { /* use SpliceType_3_PRIME */
-        SplicePredictor_add_data_primate_3SS(sp);
+        SplicePredictor_add_3SS_data(sp, sas->splice3_data_path);
         }
     if((type == SpliceType_ss5_reverse)
     || (type == SpliceType_ss3_reverse)){
@@ -174,7 +287,7 @@ SplicePredictor *SplicePredictor_create(SpliceType type,
              }
          sp->model_data[i][4] = 0.0;
          }
-    if(gtag_only)
+    if(sas->force_gtag)
         sp->gtag_only = SplicePredictor_GTAGonly_create(sp);
     else
         sp->gtag_only = NULL;
@@ -292,16 +405,12 @@ gfloat SplicePredictor_get_max_score(SplicePredictor *sp){
     return score;
     }
 
-SplicePredictorSet *SplicePredictorSet_create(gboolean gtag_only){
+SplicePredictorSet *SplicePredictorSet_create(void){
     register SplicePredictorSet *sps = g_new(SplicePredictorSet, 1);
-    sps->ss5_forward = SplicePredictor_create(SpliceType_ss5_forward,
-                                              gtag_only);
-    sps->ss5_reverse = SplicePredictor_create(SpliceType_ss5_reverse,
-                                              gtag_only);
-    sps->ss3_forward = SplicePredictor_create(SpliceType_ss3_forward,
-                                              gtag_only);
-    sps->ss3_reverse = SplicePredictor_create(SpliceType_ss3_reverse,
-                                              gtag_only);
+    sps->ss5_forward = SplicePredictor_create(SpliceType_ss5_forward);
+    sps->ss5_reverse = SplicePredictor_create(SpliceType_ss5_reverse);
+    sps->ss3_forward = SplicePredictor_create(SpliceType_ss3_forward);
+    sps->ss3_reverse = SplicePredictor_create(SpliceType_ss3_reverse);
     return sps;
     }
 
@@ -341,6 +450,7 @@ static SparseCache_Page *SplicePrediction_cache_fill_func(gint start,
                      +(spn->sp->model_length-spn->sp->model_splice_after))
                - seq_start;
     page->data = g_new(gint, pred_length);
+    page->data_size = sizeof(gint)*pred_length;
     seq = spn->get_seq_func(seq_start, seq_length, spn->user_data);
     SplicePredictor_predict_array_int(spn->sp, seq, seq_length,
                                 pred_start-seq_start, pred_length, page->data);
@@ -368,7 +478,7 @@ SplicePrediction *SplicePrediction_create(SplicePredictor *sp, gint len,
     } else {
         spn->single = NULL;
         spn->cache = SparseCache_create(len,
-                         SplicePrediction_cache_fill_func, NULL, spn);
+                         SplicePrediction_cache_fill_func, NULL, NULL, spn);
         }
     return spn;
     }
